#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_override_categorization
short_description: Manages override-categorization objects on Checkpoint over Web Services API
description:
  - Manages override-categorization objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  url:
    description:
      - The URL for which we want to update the category and risk definitions, the URL and the object name are the same for Override Categorization.
    type: str
    required: True
  url_defined_as_regular_expression:
    description:
      - States whether the URL is defined as a Regular Expression or not.
    type: bool
  new_primary_category:
    description:
      - Uid or name of the primary category based on its most defining aspect.
    type: str
  risk:
    description:
      - States the override categorization risk.
    type: str
    choices: ['unknown', 'very_low', 'low', 'medium', 'high', 'critical']
  additional_categories:
    description:
      - Uid or name of the categories to override in the Application and URL Filtering or Threat Prevention.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
  new_url:
    description:
      - New name of the object.
    type: str
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-override-categorization
  cp_mgmt_override_categorization:
    new_primary_category: Botnets
    risk: low
    state: present
    url: newOverride

- name: set-override-categorization
  cp_mgmt_override_categorization:
    risk: high
    state: present
    url: newOverride

- name: delete-override-categorization
  cp_mgmt_override_categorization:
    state: absent
    url: newOverride
"""

RETURN = """
cp_mgmt_override_categorization:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        url=dict(type='str', required=True),
        url_defined_as_regular_expression=dict(type='bool'),
        new_primary_category=dict(type='str'),
        risk=dict(type='str', choices=['unknown', 'very_low', 'low', 'medium', 'high', 'critical']),
        additional_categories=dict(type='list', elements='str'),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool'),
        new_url=dict(type='str')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'override-categorization'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sites_time_zone_settings
short_description: Resource module for Sites Time Zone Settings
description:
- This module represents an alias of the module sites_time_zone_settings_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Site Id.
    type: str
  timeZone:
    description: Sites Time Zone Settings's timeZone.
    suboptions:
      identifier:
        description: Time zone that corresponds to the site's physical location. The
          site time zone is used when scheduling device provisioning and updates. Example
          GMT.
        type: str
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings SetTimeZoneForASiteV1
  description: Complete reference of the SetTimeZoneForASiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!set-time-zone-for-a-site
notes:
  - SDK Method used are
    network_settings.NetworkSettings.set_time_zone_for_a_site_v1,

  - Paths used are
    put /dna/intent/api/v1/sites/{id}/timeZoneSettings,
  - It should be noted that this module is an alias of sites_time_zone_settings_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sites_time_zone_settings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    id: string
    timeZone:
      identifier: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_publicipaddress

version_added: "0.1.0"

short_description: Manage Azure Public IP Addresses

description:
    - Create, update and delete a Public IP address.
    - Allows setting and updating the address allocation method and domain name label.
    - Use the M(azure.azcollection.azure_rm_networkinterface) module to associate a Public IP with a network interface.

options:
    resource_group:
        description:
            - Name of resource group with which the Public IP is associated.
        required: true
        type: str
    allocation_method:
        description:
            - Control whether the assigned Public IP remains permanently assigned to the object.
            - If not set to C(Static), the IP address may changed anytime an associated virtual machine is power cycled.
        type: str
        choices:
            - dynamic
            - static
            - Static
            - Dynamic
        default: dynamic
    domain_name:
        description:
            - The customizable portion of the FQDN assigned to public IP address. This is an explicit setting.
            - If no value is provided, any existing value will be removed on an existing public IP.
        type: str
        aliases:
            - domain_name_label
    reverse_fqdn:
        description:
            - The reverse FQDN.
            - A user-visible, fully qualified domain name that resolves to this public IP address.
            - If the reverseFqdn is specified, then a PTR DNS record is created pointing from the IP address in the in-addr.arpa domain to the reverse FQDN.
        type: str
    name:
        description:
            - Name of the Public IP.
        type: str
        required: true
    state:
        description:
            - Assert the state of the Public IP. Use C(present) to create or update a and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    sku:
        description:
            - The public IP address SKU.
            - When I(version=ipv6), if I(sku=standard) then set I(allocation_method=static).
            - When I(version=ipv4), if I(sku=standard) then set I(allocation_method=static).
        type: str
        choices:
            - basic
            - standard
            - Basic
            - Standard
    ip_tags:
        description:
            - List of IpTag associated with the public IP address.
            - Each element should contain type:value pair.
        type: list
        elements: dict
        suboptions:
            type:
                description:
                    - Sets the ip_tags type.
                type: str
                required: true
            value:
                description:
                    - Sets the ip_tags value.
                type: str
                required: true
    idle_timeout:
        description:
            - Idle timeout in minutes.
        type: int
    version:
        description:
            - The public IP address version.
        type: str
        choices:
            - ipv4
            - ipv6
        default: ipv4
    zones:
        description:
            - A list of availability zones denoting the IP allocated for the resource needs to come from.
        type: list
        elements: str
        choices:
            - '1'
            - '2'
            - '3'

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)
'''

EXAMPLES = '''
- name: Create a public ip address
  azure_rm_publicipaddress:
    resource_group: myResourceGroup
    name: my_public_ip
    allocation_method: static
    domain_name: foobar

- name: Delete public ip
  azure_rm_publicipaddress:
    resource_group: myResourceGroup
    name: my_public_ip
    state: absent
'''

RETURN = '''
state:
    description:
        - Facts about the current state of the object.
    returned: always
    type: complex
    contains:
        dns_settings:
            description:
                - The FQDN of the DNS record associated with the public IP address.
            returned: always
            type: dict
            sample: {
            "domain_name_label": "ansible-b57dc95985712e45eb8b9c2e",
            "fqdn": "ansible-b57dc95985712e45eb8b9c2e.eastus.cloudapp.azure.com",
            "reverse_fqdn": "ansible-b57dc95985712e45eb8b9c2f.eastus.cloudapp.azure.com"
        }
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: "W/'1905ee13-7623-45b1-bc6b-4a12b2fb9d15'"
        idle_timeout_in_minutes:
            description:
                - The idle timeout of the public IP address.
            returned: always
            type: int
            sample: 4
        ip_address:
            description:
                - The Public IP Prefix this Public IP Address should be allocated from.
            returned: always
            type: str
            sample: 52.160.103.93
        location:
            description:
                - Resource location.
            returned: always
            type: str
            example: eastus
        name:
            description:
                - Name of the Public IP Address.
            returned: always
            type: str
            example: publicip002
        provisioning_state:
            description:
                - The provisioning state of the Public IP resource.
            returned: always
            type: str
            example: Succeeded
        public_ip_allocation_method:
             description:
                 - The public IP allocation method.
             returned: always
             type: str
             sample: static
        public_ip_address_version:
             description:
                 - The public IP address version.
             returned: always
             type: str
             sample: ipv4
        sku:
            description:
                - The public IP address SKU.
            returned: always
            type: str
            sample: Basic
        tags:
            description:
                - The resource tags.
            returned: always
            type: dict
            sample: {
                "delete": "on-exit",
                "testing": "testing"
                }
        type:
            description:
                - Type of the resource.
            returned: always
            type: str
            sample: "Microsoft.Network/publicIPAddresses"
        zones:
            description:
                - A list of availability zones denoting the IP allocated for the resource needs to come from.
            returned: always
            type: list
            sample: ['1', '2']
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils._text import to_native

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def pip_to_dict(pip):
    result = dict(
        name=pip.name,
        type=pip.type,
        location=pip.location,
        tags=pip.tags,
        public_ip_allocation_method=pip.public_ip_allocation_method.lower(),
        public_ip_address_version=pip.public_ip_address_version.lower(),
        dns_settings=dict(),
        ip_address=pip.ip_address,
        idle_timeout_in_minutes=pip.idle_timeout_in_minutes,
        provisioning_state=pip.provisioning_state,
        etag=pip.etag,
        sku=pip.sku.name,
        zones=pip.zones
    )
    if pip.dns_settings:
        result['dns_settings']['domain_name_label'] = pip.dns_settings.domain_name_label
        result['dns_settings']['fqdn'] = pip.dns_settings.fqdn
        result['dns_settings']['reverse_fqdn'] = pip.dns_settings.reverse_fqdn
    if pip.ip_tags:
        result['ip_tags'] = [dict(type=to_native(x.ip_tag_type), value=to_native(x.tag)) for x in pip.ip_tags]
    return result


ip_tag_spec = dict(
    type=dict(type='str', required=True),
    value=dict(type='str', required=True)
)


class AzureRMPublicIPAddress(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            version=dict(type='str', default='ipv4', choices=['ipv4', 'ipv6']),
            allocation_method=dict(type='str', default='dynamic', choices=['Dynamic', 'Static', 'dynamic', 'static']),
            domain_name=dict(type='str', aliases=['domain_name_label']),
            reverse_fqdn=dict(type='str'),
            sku=dict(type='str', choices=['Basic', 'Standard', 'basic', 'standard']),
            ip_tags=dict(type='list', elements='dict', options=ip_tag_spec),
            idle_timeout=dict(type='int'),
            zones=dict(type='list', elements='str', choices=['1', '2', '3'])
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.state = None
        self.tags = None
        self.zones = None
        self.allocation_method = None
        self.domain_name = None
        self.reverse_fqdn = None
        self.sku = None
        self.version = None
        self.ip_tags = None
        self.idle_timeout = None

        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureRMPublicIPAddress, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                     supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        results = dict()
        changed = False
        pip = None

        # capitalize the sku and allocation_method. basic => Basic, Basic => Basic.
        self.allocation_method = self.allocation_method.capitalize() if self.allocation_method else None
        self.sku = self.sku.capitalize() if self.sku else None
        self.version = 'IPv4' if self.version == 'ipv4' else 'IPv6'

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        try:
            self.log("Fetch public ip {0}".format(self.name))
            pip = self.network_client.public_ip_addresses.get(self.resource_group, self.name)
            self.check_provisioning_state(pip, self.state)
            self.log("PIP {0} exists".format(self.name))
            if self.state == 'present':
                results = pip_to_dict(pip)
                domain_lable = results['dns_settings'].get('domain_name_label')
                if self.domain_name is not None and ((self.domain_name or domain_lable) and self.domain_name != domain_lable):
                    self.log('CHANGED: domain_name_label')
                    changed = True
                    results['dns_settings']['domain_name_label'] = self.domain_name

                if self.reverse_fqdn is not None and self.reverse_fqdn != results['dns_settings'].get('reverse_fqdn'):
                    changed = True
                    self.log('CHANGED: reverse_fqdn')
                else:
                    self.reverse_fqdn = results['dns_settings'].get('reverse_fqdn')

                if self.allocation_method.lower() != results['public_ip_allocation_method'].lower():
                    self.log("CHANGED: allocation_method")
                    changed = True
                    results['public_ip_allocation_method'] = self.allocation_method

                if self.sku and self.sku != results['sku']:
                    self.log("CHANGED: sku")
                    changed = True
                    results['sku'] = self.sku

                if self.version.lower() != results['public_ip_address_version'].lower():
                    self.log("CHANGED: version")
                    changed = True
                    results['public_ip_address_version'] = self.version

                if self.idle_timeout and self.idle_timeout != results['idle_timeout_in_minutes']:
                    self.log("CHANGED: idle_timeout")
                    changed = True
                    results['idle_timeout_in_minutes'] = self.idle_timeout

                if self.zones and self.zones != results['zones']:
                    self.log("Zones defined do not same with existing zones")
                    changed = False
                    self.fail("ResourceAvailabilityZonesCannotBeModified: defines is {0}, existing is {1}".format(self.zones, results['zones']))

                if str(self.ip_tags or []) != str(results.get('ip_tags') or []):
                    self.log("CHANGED: ip_tags")
                    changed = True
                    results['ip_tags'] = self.ip_tags

                update_tags, results['tags'] = self.update_tags(results['tags'])
                if update_tags:
                    changed = True

            elif self.state == 'absent':
                self.log("CHANGED: public ip {0} exists but requested state is 'absent'".format(self.name))
                changed = True
        except ResourceNotFoundError:
            self.log('Public ip {0} does not exist'.format(self.name))
            if self.state == 'present':
                self.log("CHANGED: pip {0} does not exist but requested state is 'present'".format(self.name))
                changed = True

        self.results['state'] = results
        self.results['changed'] = changed

        if self.check_mode:
            return results

        if changed:
            if self.state == 'present':
                if not pip:
                    self.log("Create new Public IP {0}".format(self.name))
                    pip = self.network_models.PublicIPAddress(
                        location=self.location,
                        public_ip_address_version=self.version,
                        public_ip_allocation_method=self.allocation_method,
                        sku=self.network_models.PublicIPAddressSku(name=self.sku) if self.sku else None,
                        idle_timeout_in_minutes=self.idle_timeout if self.idle_timeout and self.idle_timeout > 0 else None,
                        zones=self.zones
                    )
                    if self.ip_tags:
                        pip.ip_tags = [self.network_models.IpTag(ip_tag_type=x['type'], tag=x['value']) for x in self.ip_tags]
                    if self.tags:
                        pip.tags = self.tags
                    if self.domain_name or self.reverse_fqdn:
                        pip.dns_settings = self.network_models.PublicIPAddressDnsSettings(
                            domain_name_label=self.domain_name,
                            reverse_fqdn=self.reverse_fqdn
                        )
                else:
                    self.log("Update Public IP {0}".format(self.name))
                    pip = self.network_models.PublicIPAddress(
                        location=results['location'],
                        public_ip_allocation_method=results['public_ip_allocation_method'],
                        sku=self.network_models.PublicIPAddressSku(name=self.sku) if self.sku else None,
                        tags=results['tags'],
                        zones=results['zones']
                    )
                    if self.domain_name or self.reverse_fqdn:
                        pip.dns_settings = self.network_models.PublicIPAddressDnsSettings(
                            domain_name_label=self.domain_name,
                            reverse_fqdn=self.reverse_fqdn
                        )
                self.results['state'] = self.create_or_update_pip(pip)
            elif self.state == 'absent':
                self.log('Delete public ip {0}'.format(self.name))
                self.delete_pip()

        return self.results

    def create_or_update_pip(self, pip):
        try:
            poller = self.network_client.public_ip_addresses.begin_create_or_update(self.resource_group, self.name, pip)
            pip = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error creating or updating {0} - {1}".format(self.name, str(exc)))
        return pip_to_dict(pip)

    def delete_pip(self):
        try:
            poller = self.network_client.public_ip_addresses.begin_delete(self.resource_group, self.name)
            self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error deleting {0} - {1}".format(self.name, str(exc)))
        # Delete returns nada. If we get here, assume that all is well.
        self.results['state']['status'] = 'Deleted'
        return True


def main():
    AzureRMPublicIPAddress()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: backup_plan
version_added: 6.0.0
short_description: Manage AWS Backup Plans
description:
  - Creates, updates, or deletes AWS Backup Plans.
  - For more information see the AWS documentation for Backup plans U(https://docs.aws.amazon.com/aws-backup/latest/devguide/about-backup-plans.html).
author:
  - Kristof Imre Szabo (@krisek)
  - Alina Buzachis (@alinabuzachis)
  - Helen Bailey (@hakbailey)
options:
  state:
    description:
      - Create/update or delete a backup plan.
    type: str
    default: present
    choices: ['present', 'absent']
  backup_plan_name:
    description:
      - The display name of a backup plan. Must contain 1 to 50 alphanumeric or '-_.' characters.
    type: str
    required: true
    aliases: ['name']
  rules:
    description:
      - An array of BackupRule objects, each of which specifies a scheduled task that is used to back up a selection of resources.
      - Required when O(state=present).
    type: list
    elements: dict
    suboptions:
      rule_name:
        description: Name of the rule.
        type: str
        required: true
      target_backup_vault_name:
        description: Name of the Backup Vault this rule should target.
        type: str
        required: true
      schedule_expression:
        description: A CRON expression in UTC specifying when Backup initiates a backup
          job. AWS default is used if not supplied.
        type: str
        default: 'cron(0 5 ? * * *)'
      start_window_minutes:
        description:
          - A value in minutes after a backup is scheduled before a job will be
            canceled if it doesn't start successfully. If this value is included, it
            must be at least 60 minutes to avoid errors.
          - AWS default if not supplied is V(480).
        type: int
        default: 480
      completion_window_minutes:
        description:
          - A value in minutes after a backup job is successfully started before it
            must be completed or it will be canceled by Backup.
          - AWS default if not supplied is V(10080).
        type: int
        default: 10080
      lifecycle:
        description:
          - The lifecycle defines when a protected resource is transitioned to cold
            storage and when it expires. Backup will transition and expire backups
            automatically according to the lifecycle that you define.
          - Backups transitioned to cold storage must be stored in cold storage for a
            minimum of 90 days. Therefore, the "retention" setting must be 90 days
            greater than the "transition to cold after days" setting. The "transition
            to cold after days" setting cannot be changed after a backup has been
            transitioned to cold.
        type: dict
        suboptions:
          move_to_cold_storage_after_days:
            description: Specifies the number of days after creation that a recovery point is moved to cold storage.
            type: int
          delete_after_days:
            description: Specifies the number of days after creation that a recovery
              point is deleted. Must be greater than V(90) days plus
              O(rules.lifecycle.move_to_cold_storage_after_days).
            type: int
      recovery_point_tags:
        description: To help organize your resources, you can assign your own metadata to the resources that you create.
        type: dict
      copy_actions:
        description: An array of copy_action objects, which contains the details of the copy operation.
        type: list
        elements: dict
        suboptions:
          destination_backup_vault_arn:
            description: An Amazon Resource Name (ARN) that uniquely identifies the destination backup vault for the copied backup.
            type: str
            required: true
          lifecycle:
            description:
              - Contains an array of Transition objects specifying how long in days
                before a recovery point transitions to cold storage or is deleted.
              - Backups transitioned to cold storage must be stored in cold storage for
                a minimum of 90 days. Therefore, on the console, the "retention"
                setting must be 90 days greater than the "transition to cold after
                days" setting. The "transition to cold after days" setting cannot be
                changed after a backup has been transitioned to cold.
            type: dict
            suboptions:
              move_to_cold_storage_after_days:
                description: Specifies the number of days after creation that a
                  recovery point is moved to cold storage.
                type: int
              delete_after_days:
                description: Specifies the number of days after creation that a
                  recovery point is deleted. Must be greater than V(90) days plus
                   O(rules.copy_actions.lifecycle.move_to_cold_storage_after_days).
                type: int
      enable_continuous_backup:
        description:
          - Specifies whether Backup creates continuous backups. True causes Backup to
            create continuous backups capable of point-in-time restore (PITR). False
            (or not specified) causes Backup to create snapshot backups.
          - AWS default if not supplied is V(false).
        type: bool
        default: false
      schedule_expression_timezone:
        description:
          - This is the timezone in which the schedule expression is set.
          - By default, ScheduleExpressions are in UTC. You can modify this to a specified timezone.
          - This option requires botocore >= 1.31.36.
        type: str
        default: "Etc/UTC"
        required: false
        version_added: 7.3.0
  advanced_backup_settings:
    description:
      - Specifies a list of advanced backup settings for each resource type.
      - These settings are only available for Windows Volume Shadow Copy Service (VSS) backup jobs.
    required: false
    type: list
    elements: dict
    suboptions:
      resource_type:
        description:
          - Specifies an object containing resource type and backup options.
          - The only supported resource type is Amazon EC2 instances with Windows Volume Shadow Copy Service (VSS).
        type: str
        choices: ['EC2']
      backup_options:
        description:
          - Specifies the backup option for a selected resource.
          - This option is only available for Windows VSS backup jobs.
        type: dict
        suboptions:
          WindowsVSS:
            description: Enable or disable WindowsVSS backup option.
            type: str
            choices: ['enabled', 'disabled']
  creator_request_id:
    description: Identifies the request and allows failed requests to be retried
      without the risk of running the operation twice. If the request includes a
      CreatorRequestId that matches an existing backup plan, that plan is returned.
    type: str
  tags:
    description: To help organize your resources, you can assign your own metadata to
      the resources that you create. Each tag is a key-value pair. The specified tags
      are assigned to all backups created with this plan.
    type: dict
    aliases: ['resource_tags', 'backup_plan_tags']

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
  - amazon.aws.tags
"""

EXAMPLES = r"""
- name: Create an AWSbackup plan
  amazon.aws.backup_plan:
    state: present
    backup_plan_name: elastic
    rules:
      - rule_name: daily
        advanced_backup_settings:
          - resource_type: "EC2"
            backup_options:
              WindowsVSS: enabled
        target_backup_vault_name: "{{ backup_vault_name }}"
        schedule_expression: 'cron(0 5 ? * * *)'
        start_window_minutes: 60
        completion_window_minutes: 1440
- name: Delete an AWS Backup plan
  amazon.aws.backup_plan:
    backup_plan_name: elastic
    state: absent
"""

RETURN = r"""
exists:
  description: Whether the resource exists.
  returned: always
  type: bool
  sample: true
backup_plan_arn:
  description: ARN of the backup plan.
  returned: always
  type: str
  sample: arn:aws:backup:eu-central-1:111122223333:backup-plan:1111f877-1ecf-4d79-9718-a861cd09df3b
backup_plan_id:
  description: ID of the backup plan.
  returned: always
  type: str
  sample: "1111f877-1ecf-4d79-9718-a861cd09df3b"
backup_plan_name:
  description: Name of the backup plan.
  returned: always
  type: str
  sample: elastic
creation_date:
  description: Creation date of the backup plan.
  returned: on create/update
  type: str
  sample: '2023-01-24T10:08:03.193000+01:00'
deletion_date:
  description: Date the backup plan was deleted.
  returned: on delete
  type: str
  sample: '2023-05-05T16:24:51.987000-04:00'
version_id:
  description: Version ID of the backup plan.
  returned: always
  type: str
  sample: ODM3MjVjNjItYWFkOC00NjExLWIwZTYtZDNiNGI5M2I0ZTY1
backup_plan:
  description: Backup plan details.
  returned: on create/update
  type: dict
  contains:
    backup_plan_name:
      description: The display name of a backup plan.
      returned: always
      type: str
      sample: elastic
    rules:
      description:
        - An array of BackupRule objects, each of which specifies a scheduled task that is used to back up a selection of resources.
      returned: always
      type: list
      elements: dict
      contains:
        rule_name:
          description: A display name for a backup rule.
          returned: always
          type: str
          sample: "daily"
        target_backup_vault_name:
          description: The name of a logical container where backups are stored.
          returned: always
          type: str
          sample: "09da67966fd5-backup-vault"
        schedule_expression:
          description: A cron expression in UTC specifying when Backup initiates a backup job.
          returned: always
          type: str
          sample: "cron(0 5 ? * * *)"
        start_window_minutes:
          description:
            - A value in minutes after a backup is scheduled before a job will be canceled if it
              doesn't start successfully.
          type: int
          returned: always
          sample: 480
        completion_window_minutes:
          description:
            - A value in minutes after a backup job is successfully started before it must be
              completed or it will be canceled by Backup.
          type: int
          returned: always
          sample: 10080
        lifecycle:
          description:
            - Defines when a protected resource is transitioned to cold storage and when it expires.
          type: dict
          returned: when configured.
          sample: {
                    "delete_after_days": 100,
                    "move_to_cold_storage_after_days": 10
                  }
        recovery_point_tags:
          description:
            - An array of key-value pair strings that are assigned to resources that are associated with
              this rule when restored from backup.
          type: dict
          returned: when configured.
          sample: {
                    "Tagkey1": "TagValue1",
                    "Tagkey2": "TagValue2"
                  }
        rule_id:
          description:
            - Uniquely identifies a rule that is used to schedule the backup of a selection of resources.
          type: str
          returned: always
          sample: "973621ef-d863-41ef-b5c3-9e943a64ad0c"
        copy_actions:
          description: An array of CopyAction objects, which contains the details of the copy operation.
          returned: when configured.
          type: list
          elements: dict
          sample: [
                    {
                      "destination_backup_vault_arn": "arn:aws:backup:us-west-2:123456789012:backup-vault:my-test-vault",
                      "lifecycle": {
                          "delete_after_days": 100,
                          "move_to_cold_storage_after_days": 10
                      }
                    }
                  ]
        enable_continous_backup:
          description: Specifies whether Backup creates continuous backups.
          type: bool
          returned: always
          sample: false
        schedule_expression_timezone:
          description:
            - This is the timezone in which the schedule expression is set.
            - This information is returned for botocore versions >= 1.31.36.
          type: str
          returned: when botocore >= 1.31.36
          sample: "Etc/UTC"
          version_added: 7.3.0
    advanced_backup_settings:
      description: Advanced backup settings of the backup plan.
      returned: when configured.
      type: list
      elements: dict
      sample: [
                {
                  "backup_options": {
                      "windows_vss": "enabled"
                  },
                  "resource_type": "EC2"
                }
              ]
      contains:
        resource_type:
          description: Resource type of the advanced settings.
          type: str
          sample: "EC2"
        backup_options:
          description: Backup options of the advanced settings.
          type: dict
          sample: {
                    "windows_vss": "enabled"
                  }
    tags:
      description: Tags of the backup plan.
      returned: on create/update
      type: str
      sample: {
                "TagKey1": "TagValue1",
                "TagKey2": "TagValue2"
              }
"""

import json
from datetime import datetime
from typing import Optional

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.backup import get_plan_details
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags
from ansible_collections.amazon.aws.plugins.module_utils.transformation import scrub_none_parameters

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

ARGUMENT_SPEC = dict(
    state=dict(type="str", choices=["present", "absent"], default="present"),
    backup_plan_name=dict(required=True, type="str", aliases=["name"]),
    rules=dict(
        type="list",
        elements="dict",
        options=dict(
            rule_name=dict(required=True, type="str"),
            target_backup_vault_name=dict(required=True, type="str"),
            schedule_expression=dict(type="str", default="cron(0 5 ? * * *)"),
            start_window_minutes=dict(type="int", default=480),
            completion_window_minutes=dict(type="int", default=10080),
            schedule_expression_timezone=dict(type="str", default="Etc/UTC"),
            lifecycle=dict(
                type="dict",
                options=dict(
                    move_to_cold_storage_after_days=dict(type="int"),
                    delete_after_days=dict(type="int"),
                ),
            ),
            recovery_point_tags=dict(type="dict"),
            copy_actions=dict(
                type="list",
                elements="dict",
                options=dict(
                    destination_backup_vault_arn=dict(required=True, type="str"),
                    lifecycle=dict(
                        type="dict",
                        options=dict(
                            move_to_cold_storage_after_days=dict(type="int"),
                            delete_after_days=dict(type="int"),
                        ),
                    ),
                ),
            ),
            enable_continuous_backup=dict(type="bool", default=False),
        ),
    ),
    advanced_backup_settings=dict(
        type="list",
        elements="dict",
        options=dict(
            resource_type=dict(type="str", choices=["EC2"]),
            backup_options=dict(
                type="dict",
                options=dict(WindowsVSS=dict(type="str", choices=["enabled", "disabled"])),
            ),
        ),
    ),
    creator_request_id=dict(type="str"),
    tags=dict(type="dict", aliases=["backup_plan_tags", "resource_tags"]),
    purge_tags=dict(default=True, type="bool"),
)

REQUIRED_IF = [
    ("state", "present", ["backup_plan_name", "rules"]),
    ("state", "absent", ["backup_plan_name"]),
]

SUPPORTS_CHECK_MODE = True


def format_client_params(
    module: AnsibleAWSModule,
    plan: dict,
    tags: Optional[dict] = None,
    backup_plan_id: Optional[str] = None,
    operation: Optional[str] = None,
) -> dict:
    """
    Formats plan details to match boto3 backup client param expectations.

    module : AnsibleAWSModule object
    plan: Dict of plan details including name, rules, and advanced settings
    tags: Dict of plan tags
    backup_plan_id: ID of backup plan to update, only needed for update operation
    operation: Operation to add specific params for, either create or update
    """
    params = {
        "BackupPlan": snake_dict_to_camel_dict(
            {k: v for k, v in plan.items() if v != "backup_plan_name"},
            capitalize_first=True,
        )
    }

    if operation == "create":  # Add create-specific params
        if tags:
            params["BackupPlanTags"] = tags
        creator_request_id = module.params["creator_request_id"]
        if creator_request_id:
            params["CreatorRequestId"] = creator_request_id

    elif operation == "update":  # Add update-specific params
        params["BackupPlanId"] = backup_plan_id

    return params


def format_check_mode_response(plan_name: str, plan: dict, tags: dict, delete: bool = False) -> dict:
    """
    Formats plan details in check mode to match result expectations.

    plan_name: Name of backup plan
    plan: Dict of plan details including name, rules, and advanced settings
    tags: Optional dict of plan tags
    delete: Whether the response is for a delete action
    """
    timestamp = datetime.now().isoformat()
    if delete:
        return {
            "backup_plan_name": plan_name,
            "backup_plan_id": "",
            "backup_plan_arn": "",
            "deletion_date": timestamp,
            "version_id": "",
        }
    else:
        return {
            "backup_plan_name": plan_name,
            "backup_plan_id": "",
            "backup_plan_arn": "",
            "creation_date": timestamp,
            "version_id": "",
            "backup_plan": {
                "backup_plan_name": plan_name,
                "rules": plan["rules"],
                "advanced_backup_settings": plan["advanced_backup_settings"],
                "tags": tags,
            },
        }


def create_backup_plan(module: AnsibleAWSModule, client, create_params: dict) -> dict:
    """
    Creates a backup plan.

    module : AnsibleAWSModule object
    client : boto3 backup client connection object
    create_params : The boto3 backup client parameters to create a backup plan
    """
    try:
        response = client.create_backup_plan(**create_params)
    except (
        BotoCoreError,
        ClientError,
    ) as err:
        module.fail_json_aws(err, msg="Failed to create backup plan {err}")
    return response


def plan_update_needed(existing_plan: dict, new_plan: dict) -> bool:
    """
    Determines whether existing and new plan rules/settings match.

    existing_plan: Dict of existing plan details including rules and advanced settings,
        in snake-case format
    new_plan: Dict of existing plan details including rules and advanced settings, in
        snake-case format
    """
    update_needed = False

    # Check whether rules match
    existing_rules = json.dumps(
        [{key: val for key, val in rule.items() if key != "rule_id"} for rule in existing_plan["backup_plan"]["rules"]],
        sort_keys=True,
    )
    new_rules = json.dumps(new_plan["rules"], sort_keys=True)
    if not existing_rules or existing_rules != new_rules:
        update_needed = True

    # Check whether advanced backup settings match
    existing_advanced_backup_settings = json.dumps(
        existing_plan["backup_plan"].get("advanced_backup_settings", []),
        sort_keys=True,
    )
    new_advanced_backup_settings = json.dumps(new_plan.get("advanced_backup_settings", []), sort_keys=True)
    if existing_advanced_backup_settings != new_advanced_backup_settings:
        update_needed = True

    return update_needed


def update_backup_plan(module: AnsibleAWSModule, client, update_params: dict) -> dict:
    """
    Updates a backup plan.

    module : AnsibleAWSModule object
    client : boto3 backup client connection object
    update_params : The boto3 backup client parameters to update a backup plan
    """
    try:
        response = client.update_backup_plan(**update_params)
    except (
        BotoCoreError,
        ClientError,
    ) as err:
        module.fail_json_aws(err, msg="Failed to update backup plan {err}")
    return response


def tag_backup_plan(
    module: AnsibleAWSModule,
    client,
    new_tags: Optional[dict],
    plan_arn: str,
    current_tags: Optional[dict] = None,
):
    """
    Creates, updates, and/or removes tags on a Backup Plan resource.

    module : AnsibleAWSModule object
    client : boto3 client connection object
    new_tags : Dict of tags converted from ansible_dict to boto3 list of dicts
    plan_arn : The ARN of the Backup Plan to operate on
    curr_tags : Dict of the current tags on resource, if any
    """

    if not new_tags and not current_tags:
        return False

    if module.check_mode:
        return True

    new_tags = new_tags or {}
    current_tags = current_tags or {}
    tags_to_add, tags_to_remove = compare_aws_tags(current_tags, new_tags, purge_tags=module.params["purge_tags"])

    if not tags_to_add and not tags_to_remove:
        return False

    if tags_to_remove:
        try:
            client.untag_resource(ResourceArn=plan_arn, TagKeyList=tags_to_remove)
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to remove tags from the plan")

    if tags_to_add:
        try:
            client.tag_resource(ResourceArn=plan_arn, Tags=tags_to_add)
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to add tags to the plan")

    return True


def delete_backup_plan(module: AnsibleAWSModule, client, backup_plan_id: str) -> dict:
    """
    Deletes a Backup Plan

    module : AnsibleAWSModule object
    client : boto3 backup client connection object
    backup_plan_id : ID (*not* name or ARN) of Backup plan to delete
    """
    try:
        response = client.delete_backup_plan(BackupPlanId=backup_plan_id)
    except (BotoCoreError, ClientError) as err:
        module.fail_json_aws(err, msg="Failed to delete the Backup Plan")
    return response


def main():
    module = AnsibleAWSModule(
        argument_spec=ARGUMENT_SPEC,
        required_if=REQUIRED_IF,
        supports_check_mode=SUPPORTS_CHECK_MODE,
    )

    # Set initial result values
    result = dict(changed=False, exists=False)

    # Get supplied params from module
    client = module.client("backup")
    state = module.params["state"]
    plan_name = module.params["backup_plan_name"]

    plan = {
        "backup_plan_name": module.params["backup_plan_name"],
        "rules": [scrub_none_parameters(rule) for rule in module.params["rules"] or []],
        "advanced_backup_settings": [
            scrub_none_parameters(setting) for setting in module.params["advanced_backup_settings"] or []
        ],
    }

    if module.params["rules"]:
        for each in plan["rules"]:
            if not module.botocore_at_least("1.31.36"):
                module.warn(
                    "schedule_expression_timezone requires botocore >= 1.31.36. schedule_expression_timezone will be ignored."
                )
                each.pop("schedule_expression_timezone")

    tags = module.params["tags"]

    # Get existing backup plan details and ID if present
    existing_plan = get_plan_details(module, client, plan_name)
    if existing_plan:
        existing_plan_id = existing_plan[0]["backup_plan_id"]
        existing_plan = existing_plan[0]
    else:
        existing_plan = existing_plan_id = None

    if state == "present":  # Create or update plan
        if existing_plan_id is None:  # Plan does not exist, create it
            if module.check_mode:  # Use supplied params as result data in check mode
                backup_plan = format_check_mode_response(plan_name, plan, tags)
            else:
                client_params = format_client_params(module, plan, tags=tags, operation="create")
                response = create_backup_plan(module, client, client_params)
                backup_plan = get_plan_details(module, client, plan_name)[0]
            result["exists"] = True
            result["changed"] = True
            result.update(backup_plan)

        else:  # Plan exists, update as needed
            result["exists"] = True
            if plan_update_needed(existing_plan, plan):
                if not module.check_mode:
                    client_params = format_client_params(
                        module,
                        plan,
                        backup_plan_id=existing_plan_id,
                        operation="update",
                    )
                    update_backup_plan(module, client, client_params)
                result["changed"] = True
            if tag_backup_plan(
                module,
                client,
                tags,
                existing_plan["backup_plan_arn"],
                existing_plan["tags"],
            ):
                result["changed"] = True
            if module.check_mode:
                backup_plan = format_check_mode_response(plan_name, plan, tags)
            else:
                backup_plan = get_plan_details(module, client, plan_name)[0]
            result.update(backup_plan)

    elif state == "absent":  # Delete plan
        if existing_plan_id is None:  # Plan does not exist, can't delete it
            module.debug(msg=f"Backup plan {plan_name} not found.")
        else:  # Plan exists, delete it
            if module.check_mode:
                response = format_check_mode_response(plan_name, existing_plan, tags, True)
            else:
                response = delete_backup_plan(module, client, existing_plan_id)
            result["changed"] = True
            result["exists"] = False
            result.update(camel_dict_to_snake_dict(response))

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 GuopengLin, (@t-glin)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_route_info
version_added: '2.0.0'
short_description: Get Route info
description:
    - Get info of Route.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    route_table_name:
        description:
            - The name of the route table.
        required: true
        type: str
    name:
        description:
            - The name of the route.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - GuopengLin (@t-glin)
    - Fred-Sun (@Fred-Sun)
    - Haiyuan Zhang (@haiyuazhang)

'''

EXAMPLES = '''
- name: Get route
  azure_rm_route_info:
    resource_group: myResourceGroup
    name: routename
    route_table_name: routetabename


- name: List routes
  azure_rm_route_info:
    resource_group: myResourceGroup
    route_table_name: routetabename
'''

RETURN = '''
routes:
    description:
        - A list of dict results where the key is the name of the Route and the values are the facts for that Route.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/routeTables/table01/routes/route01
        name:
            description:
                - The name of the resource that is unique within a resource group.
                - This name can be used to access the resource.
            returned: always
            type: str
            sample: route01
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: fda240c5-6f42-4eb0-8f05-b1599dc722fe
        address_prefix:
            description:
                - The destination CIDR to which the route applies.
            returned: always
            type: str
            sample: 10.0.0.0/25
        next_hop_type:
            description:
                - The type of Azure hop the packet should be sent to.
            returned: always
            type: str
            sample: VirtualAppliance
        next_hop_ip_address:
            description:
                - The IP address packets should be forwarded to.
                - Next hop values are only allowed in routes where the next hop type is VirtualAppliance.
            returned: always
            type: str
            sample: 10.0.0.0
        provisioning_state:
            description:
                - The provisioning state of the route resource.
            returned: always
            type: str
            sample: Succeeded

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase
try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMRouteInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            route_table_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )

        self.resource_group = None
        self.route_table_name = None
        self.name = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]
        self.tags = None

        self.mgmt_client = None
        super(AzureRMRouteInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if (self.resource_group is not None and self.route_table_name is not None and self.name is not None):
            self.results['routes'] = self.format_item(self.get())
        elif (self.resource_group is not None and
              self.route_table_name is not None):
            self.results['routes'] = self.format_item(self.list())
        return self.results

    def get(self):
        response = None

        try:
            response = self.network_client.routes.get(resource_group_name=self.resource_group,
                                                      route_table_name=self.route_table_name,
                                                      route_name=self.name)
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def list(self):
        response = None

        try:
            response = self.network_client.routes.list(resource_group_name=self.resource_group,
                                                       route_table_name=self.route_table_name)
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def format_item(self, item):
        if hasattr(item, 'as_dict'):
            return [item.as_dict()]
        else:
            result = []
            items = list(item)
            for tmp in items:
                result.append(tmp.as_dict())
            return result


def main():
    AzureRMRouteInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: event_snmp_config_v1_info
short_description: Information module for Event Snmp Config V1
description:
  - This module retrieves SNMP configuration information for events in Cisco DNA Center.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
notes:
  - Paths used are

"""

EXAMPLES = r"""
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample:
  - {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: grpc_get
version_added: "3.1.0"
author:
    - "Ganesh Nalawade (@ganeshrn)"
    - "Gomathi Selvi S (@GomathiselviS)"
short_description: Fetch configuration/state data from gRPC enabled target hosts.
description:
    - gRPC is a high performance, open-source universal RPC framework.
    - This module allows the user to fetch configuration and state data from gRPC
      enabled devices.
options:
  section:
    description:
      - This option specifies the string which acts as a filter to restrict the portions of
        the data to be retrieved from the target host device. If this option is not specified the entire
        configuration or state data is returned in response provided it is supported by target host.
    aliases:
    - filter
    type: str
  command:
    description:
      - The option specifies the command to be executed on the target host and return the response
        in result. This option is supported if the gRPC target host supports executing CLI command
        over the gRPC connection.
    type: str
  display:
    description:
      - Encoding scheme to use when serializing output from the device. The encoding scheme
        value depends on the capability of the gRPC server running on the target host.
        The values can be I(json), I(text) etc.
    type: str
  data_type:
    description:
      - The type of data that should be fetched from the target host. The value depends on the
        capability of the gRPC server running on target host. The values can be I(config), I(oper)
        etc. based on what is supported by the gRPC server. By default it will return both configuration
        and operational state data in response.
    type: str
requirements:
  - grpcio
  - protobuf
notes:
  - This module requires the gRPC system service be enabled on
    the target host being managed.
  - This module supports the use of connection=ansible.netcommon.grpc.
  - This module requires the value of 'ansible_network_os or grpc_type' configuration option (refer ansible.netcommon.grpc
    connection plugin documentation) be defined as an inventory variable.
  - Tested against iosxrv 9k version 6.1.2.
"""

EXAMPLES = """
- name: Get bgp configuration data
  grpc_get:
    section:
      Cisco-IOS-XR-ip-static-cfg:router-static:
        - null
- name: run cli command
  grpc_get:
    command: "show version"
    display: text
"""

RETURN = """
stdout:
  description: The raw string containing configuration or state data
               received from the gRPC server.
  returned: always apart from low-level errors (such as action plugin)
  type: str
  sample: '...'
stdout_lines:
  description: The value of stdout split into a list
  returned: always apart from low-level errors (such as action plugin)
  type: list
  sample: ['...', '...']
output:
  description: A dictionary representing a JSON-formatted response, if the response
               is a valid json string
  returned: when the device response is valid JSON
  type: list
  sample: |
        [{
            "Cisco-IOS-XR-ip-static-cfg:router-static": {
                "default-vrf": {
                    "address-family": {
                        "vrfipv4": {
                            "vrf-unicast": {
                                "vrf-prefixes": {
                                    "vrf-prefix": [
                                        {
                                            "prefix": "0.0.0.0",
                                            "prefix-length": 0,
                                            "vrf-route": {
                                                "vrf-next-hop-table": {
                                                    "vrf-next-hop-interface-name-next-hop-address": [
                                                        {
                                                            "interface-name": "MgmtEth0/RP0/CPU0/0",
                                                            "next-hop-address": "10.0.2.2"
                                                        }
                                                    ]
                                                }
                                            }
                                        }
                                    ]
                                }
                            }
                        }
                    }
                }
            }
        }]
"""
import json

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import ConnectionError

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list
from ansible_collections.ansible.netcommon.plugins.module_utils.network.grpc.grpc import (
    get,
    get_capabilities,
    run_cli,
)


try:
    import yaml

    HAS_YAML = True
except ImportError:
    HAS_YAML = False


def main():
    """entry point for module execution"""
    argument_spec = dict(
        section=dict(type="str", aliases=["filter"]),
        command=dict(type="str"),
        display=dict(type="str"),
        data_type=dict(type="str"),
    )

    mutually_exclusive = [["section", "command"]]
    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )
    capabilities = get_capabilities(module)

    operations = capabilities["server_capabilities"]

    if module.params["section"]:
        section = json.dumps(yaml.safe_load(module.params["section"]))
    else:
        section = None
    command = module.params["command"]
    display = module.params["display"]
    data_type = module.params["data_type"]

    supported_display = operations.get("display", [])
    if display and display not in supported_display:
        module.fail_json(
            msg="display option '%s' is not supported on this target host. Valid value is one of '%s'"
            % (display, supported_display.join(", "))
        )

    if command and not operations.get("supports_cli_command", False):
        module.fail_json(msg="command option '%s' is not supported on this target host" % command)

    supported_data_type = operations.get("data_type", [])
    if data_type and data_type not in supported_data_type:
        module.fail_json(
            msg="data_type option '%s' is not supported on this target host. Valid value is one of %s"
            % (data_type, supported_data_type.join(","))
        )

    result = {"changed": False}
    output = []
    try:
        if command:
            response, err = run_cli(module, command, display)
        else:
            response, err = get(module, section, data_type)
        try:
            output = json.loads(response)
        except ValueError:
            module.warn(to_text(err, errors="surrogate_then_replace"))

    except ConnectionError as exc:
        module.fail_json(msg=to_text(exc, errors="surrogate_then_replace"), code=exc.code)
    result["stdout"] = response

    if output:
        result["output"] = to_list(output)
    else:
        result["output"] = to_list(response)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rest_id_store
short_description: Resource module for REST Id Store
description:
- Manage operations create, update and delete of the resource REST Id Store.
- This API creates a REST ID store.
- This API deletes a REST ID store by name.
- This API deletes a REST ID store.
- This API allows the client to update a REST ID store by name.
- This API allows the client to update a REST ID store.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  description:
    description: REST Id Store's description.
    type: str
  ersRESTIDStoreAttributes:
    description: REST Id Store's ersRESTIDStoreAttributes.
    suboptions:
      headers:
        description: REST Id Store's headers.
        elements: dict
        suboptions:
          key:
            description: REST Id Store's key.
            type: str
          value:
            description: REST Id Store's value.
            type: str
        type: list
      predefined:
        description: The cloud provider connected to of the RESTIDStore. Options are
          - Azure, - Okta, - None.
        type: str
      rootUrl:
        description: Url of the root of the RESTIDStore.
        type: str
      usernameSuffix:
        description: Suffix of the username domain.
        type: str
    type: dict
  ersRESTIDStoreUserAttributes:
    description: REST Id Store's ersRESTIDStoreUserAttributes.
    suboptions:
      attributes:
        description: REST Id Store's attributes.
        elements: str
        type: list
    type: dict
  id:
    description: REST Id Store's id.
    type: str
  name:
    description: REST Id Store's name.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    restid_store.RestidStore.create_rest_id_store,
    restid_store.RestidStore.delete_rest_id_store_by_id,
    restid_store.RestidStore.delete_rest_id_store_by_name,
    restid_store.RestidStore.update_rest_id_store_by_id,
    restid_store.RestidStore.update_rest_id_store_by_name,

  - Paths used are
    post /ers/config/restidstore,
    delete /ers/config/restidstore/name/{name},
    delete /ers/config/restidstore/{id},
    put /ers/config/restidstore/name/{name},
    put /ers/config/restidstore/{id},

"""

EXAMPLES = r"""
- name: Update by name
  cisco.ise.rest_id_store:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    ersRestIDStoreAttributes:
      headers:
      - key: string
        value: string
      predefined: string
      rootUrl: string
      usernameSuffix: string
    id: string
    name: string

- name: Delete by name
  cisco.ise.rest_id_store:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    name: string

- name: Update by id
  cisco.ise.rest_id_store:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    ersRestIDStoreAttributes:
      headers:
      - key: string
        value: string
      predefined: string
      rootUrl: string
      usernameSuffix: string
    id: string
    name: string

- name: Delete by id
  cisco.ise.rest_id_store:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.rest_id_store:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    ersRestIDStoreAttributes:
      headers:
      - key: string
        value: string
      predefined: string
      rootUrl: string
      usernameSuffix: string
    ersRestIDStoreUserAttributes:
      attributes:
      - string
    name: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "ersRestIDStoreAttributes": {
        "usernameSuffix": "string",
        "rootUrl": "string",
        "predefined": "string",
        "headers": [
          {
            "key": "string",
            "value": "string"
          }
        ]
      },
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_lsv_profile
short_description: Manages lsv-profile objects on Checkpoint over Web Services API
description:
  - Manages lsv-profile objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  certificate_authority:
    description:
      - Trusted Certificate authority for establishing trust between VPN peers, identified by name or UID.
    type: str
  allowed_ip_addresses:
    description:
      - Collection of network objects identified by name or UID that represent IP addresses allowed in profile's VPN domain.
    type: list
    elements: str
  restrict_allowed_addresses:
    description:
      - Indicate whether the IP addresses allowed in the VPN Domain will be restricted or not, according to allowed-ip-addresses field.
    type: bool
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  vpn_domain:
    description:
      - peers' VPN Domain properties.
    type: dict
    suboptions:
      limit_peer_domain_size:
        description:
          - Use this parameter to limit the number of IP addresses in the VPN Domain of each peer according to the value in the max-allowed-addresses field.
        type: bool
      max_allowed_addresses:
        description:
          - Maximum number of IP addresses in the VPN Domain of each peer. This value will be enforced only when limit-peer-domain-size field is
            set to true. Select a value between 1 and 256. Default value is 256.
        type: int
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  groups:
    description:
      - Collection of group identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-lsv-profile
  cp_mgmt_lsv_profile:
    certificate_authority: dedicated_profile_certificate
    name: New lsv-profile
    state: present

- name: set-lsv-profile
  cp_mgmt_lsv_profile:
    certificate_authority: another CA
    name: existing lsv-profile
    restrict_allowed_addresses: 'false'
    state: present
    vpn_domain:
      limit_peer_domain_size: 'false'

- name: delete-lsv-profile
  cp_mgmt_lsv_profile:
    name: existing lsv-profile
    state: absent
"""

RETURN = """
cp_mgmt_lsv_profile:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        certificate_authority=dict(type='str'),
        allowed_ip_addresses=dict(type='list', elements='str'),
        restrict_allowed_addresses=dict(type='bool'),
        tags=dict(type='list', elements='str'),
        vpn_domain=dict(type='dict', options=dict(
            limit_peer_domain_size=dict(type='bool'),
            max_allowed_addresses=dict(type='int')
        )),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        groups=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'lsv-profile'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: roles
short_description: Resource module for Roles
description:
- This module represents an alias of the module roles_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Description of role.
    type: str
  resourceTypes:
    description: Roles's resourceTypes.
    elements: dict
    suboptions:
      operations:
        description: List of operations allowed for the application. Possible values
          are "gRead", "gCreate", "gUpdate", "gRemove", or some combination of these.
        elements: str
        type: list
      type:
        description: Name of the application in Cisco DNA Center System.
        type: str
    type: list
  role:
    description: Name of the role.
    type: str
  roleId:
    description: Id of the role.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles AddRoleAPIV1
  description: Complete reference of the AddRoleAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-role-api
- name: Cisco DNA Center documentation for User and Roles DeleteRoleAPIV1
  description: Complete reference of the DeleteRoleAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-role-api
- name: Cisco DNA Center documentation for User and Roles UpdateRoleAPIV1
  description: Complete reference of the UpdateRoleAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-role-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.add_role_api_v1,
    user_and_roles.UserandRoles.delete_role_api_v1,
    user_and_roles.UserandRoles.update_role_api_v1,

  - Paths used are
    post /dna/system/api/v1/role,
    delete /dna/system/api/v1/role/{roleId},
    put /dna/system/api/v1/role,
  - It should be noted that this module is an alias of roles_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.roles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    resourceTypes:
    - operations:
      - string
      type: string
    role: string

- name: Update all
  cisco.dnac.roles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    resourceTypes:
    - operations:
      - string
      type: string
    roleId: string

- name: Delete by id
  cisco.dnac.roles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    roleId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "roleId": "string",
      "message": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the eos_snmp_server module
"""


class Snmp_serverArgs(object):  # pylint: disable=R0903
    """The arg spec for the eos_snmp_server module"""

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "chassis_id": {"type": "str"},
                "communities": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"type": "str"},
                        "acl_v4": {"type": "str"},
                        "acl_v6": {"type": "str"},
                        "ro": {"type": "bool"},
                        "rw": {"type": "bool"},
                        "view": {"type": "str"},
                    },
                },
                "contact": {"type": "str"},
                "traps": {
                    "type": "dict",
                    "options": {
                        "bgp": {
                            "type": "dict",
                            "options": {
                                "arista_backward_transition": {"type": "bool"},
                                "arista_established": {"type": "bool"},
                                "backward_transition": {"type": "bool"},
                                "established": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "bridge": {
                            "type": "dict",
                            "options": {
                                "arista_mac_age": {"type": "bool"},
                                "arista_mac_learn": {"type": "bool"},
                                "arista_mac_move": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "capacity": {
                            "type": "dict",
                            "options": {
                                "arista_hardware_utilization_alert": {
                                    "type": "bool",
                                },
                                "enabled": {"type": "bool"},
                            },
                        },
                        "entity": {
                            "type": "dict",
                            "options": {
                                "arista_ent_sensor_alarm": {"type": "bool"},
                                "ent_config_change": {"type": "bool"},
                                "ent_state_oper": {"type": "bool"},
                                "ent_state_oper_disabled": {"type": "bool"},
                                "ent_state_oper_enabled": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "external_alarm": {
                            "type": "dict",
                            "options": {
                                "arista_external_alarm_asserted_notif": {
                                    "type": "bool",
                                },
                                "arista_external_alarm_deasserted_notif": {
                                    "type": "bool",
                                },
                                "enabled": {"type": "bool"},
                            },
                        },
                        "isis": {
                            "type": "dict",
                            "options": {
                                "adjacency_change": {"type": "bool"},
                                "area_mismatch": {"type": "bool"},
                                "attempt_to_exceed_max_sequence": {
                                    "type": "bool",
                                },
                                "authentication_type_failure": {
                                    "type": "bool",
                                },
                                "database_overload": {"type": "bool"},
                                "own_lsp_purge": {"type": "bool"},
                                "rejected_adjacency": {"type": "bool"},
                                "sequence_number_skip": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "lldp": {
                            "type": "dict",
                            "options": {
                                "rem_tables_change": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "mpls_ldp": {
                            "type": "dict",
                            "options": {
                                "mpls_ldp_session_down": {"type": "bool"},
                                "mpls_ldp_session_up": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "msdp": {
                            "type": "dict",
                            "options": {
                                "backward_transition": {"type": "bool"},
                                "established": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "ospf": {
                            "type": "dict",
                            "options": {
                                "if_config_error": {"type": "bool"},
                                "if_auth_failure": {"type": "bool"},
                                "if_state_change": {"type": "bool"},
                                "nbr_state_change": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "ospfv3": {
                            "type": "dict",
                            "options": {
                                "if_config_error": {"type": "bool"},
                                "if_rx_bad_packet": {"type": "bool"},
                                "if_state_change": {"type": "bool"},
                                "nbr_state_change": {"type": "bool"},
                                "nbr_restart_helper_status_change": {
                                    "type": "bool",
                                },
                                "nssa_translator_status_change": {
                                    "type": "bool",
                                },
                                "restart_status_change": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "pim": {
                            "type": "dict",
                            "options": {
                                "neighbor_loss": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "snmp": {
                            "type": "dict",
                            "options": {
                                "authentication": {"type": "bool"},
                                "link_down": {"type": "bool"},
                                "link_up": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "snmpConfigManEvent": {
                            "type": "dict",
                            "options": {
                                "arista_config_man_event": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "switchover": {
                            "type": "dict",
                            "options": {
                                "arista_redundancy_switch_over_notif": {
                                    "type": "bool",
                                },
                                "enabled": {"type": "bool"},
                            },
                        },
                        "test": {
                            "type": "dict",
                            "options": {
                                "arista_test_notification": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                        "vrrp": {
                            "type": "dict",
                            "options": {
                                "trap_new_master": {"type": "bool"},
                                "enabled": {"type": "bool"},
                            },
                        },
                    },
                },
                "engineid": {
                    "type": "dict",
                    "options": {
                        "local": {"type": "str"},
                        "remote": {
                            "type": "dict",
                            "options": {
                                "host": {"type": "str"},
                                "udp_port": {"type": "int"},
                                "id": {"type": "str"},
                            },
                        },
                    },
                },
                "extension": {
                    "type": "dict",
                    "options": {
                        "root_oid": {"type": "str"},
                        "script_location": {"type": "str"},
                        "oneshot": {"type": "bool"},
                    },
                },
                "groups": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "group": {"type": "str"},
                        "version": {
                            "type": "str",
                            "choices": ["v1", "v3", "v2c"],
                        },
                        "auth_privacy": {
                            "type": "str",
                            "choices": ["auth", "noauth", "priv"],
                        },
                        "context": {"type": "str"},
                        "notify": {"type": "str"},
                        "read": {"type": "str"},
                        "write": {"type": "str"},
                    },
                },
                "hosts": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "host": {"type": "str"},
                        "user": {"type": "str"},
                        "udp_port": {"type": "int"},
                        "informs": {"type": "bool"},
                        "traps": {"type": "bool"},
                        "version": {
                            "type": "str",
                            "choices": [
                                "1",
                                "2c",
                                "3 auth",
                                "3 noauth",
                                "3 priv",
                            ],
                        },
                        "vrf": {"type": "str"},
                    },
                },
                "acls": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "afi": {"type": "str", "choices": ["ipv4", "ipv6"]},
                        "acl": {"type": "str"},
                        "vrf": {"type": "str"},
                    },
                },
                "local_interface": {"type": "str"},
                "location": {"type": "str"},
                "notification": {"type": "int"},
                "objects": {
                    "type": "dict",
                    "options": {
                        "mac_address_tables": {"type": "bool"},
                        "route_tables": {"type": "bool"},
                    },
                },
                "qos": {"type": "int"},
                "qosmib": {"type": "int"},
                "transmit": {"type": "int"},
                "transport": {"type": "str"},
                "users": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "user": {"type": "str"},
                        "group": {"type": "str"},
                        "remote": {"type": "str"},
                        "udp_port": {"type": "int"},
                        "version": {
                            "type": "str",
                            "choices": ["v1", "v2c", "v3"],
                        },
                        "auth": {
                            "type": "dict",
                            "options": {
                                "algorithm": {"type": "str"},
                                "auth_passphrase": {
                                    "type": "str",
                                    "no_log": True,
                                },
                                "encryption": {"type": "str"},
                                "priv_passphrase": {
                                    "type": "str",
                                    "no_log": True,
                                },
                            },
                        },
                        "localized": {
                            "type": "dict",
                            "options": {
                                "engineid": {"type": "str"},
                                "algorithm": {"type": "str"},
                                "auth_passphrase": {
                                    "type": "str",
                                    "no_log": True,
                                },
                                "encryption": {"type": "str"},
                                "priv_passphrase": {
                                    "type": "str",
                                    "no_log": True,
                                },
                            },
                        },
                    },
                },
                "views": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "view": {"type": "str"},
                        "mib": {"type": "str"},
                        "action": {
                            "type": "str",
                            "choices": ["excluded", "included"],
                        },
                    },
                },
                "vrfs": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "vrf": {"type": "str"},
                        "local_interface": {"type": "str"},
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "type": "str",
            "choices": [
                "deleted",
                "merged",
                "overridden",
                "replaced",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_bulk_monitor_status_info
short_description: Information module for Network Device Bulk Monitor Status
description:
- Get Network Device Bulk Monitor Status by id.
- This API allows the client to monitor the bulk request.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  bulkid:
    description:
    - Bulkid path parameter.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    network_device.NetworkDevice.monitor_bulk_status_network_device,

  - Paths used are
    get /ers/config/networkdevice/bulk/{bulkid},

"""

EXAMPLES = r"""
- name: Get Network Device Bulk Monitor Status by id
  cisco.ise.network_device_bulk_monitor_status_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    bulkid: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "bulkId": "string",
      "mediaType": "string",
      "executionStatus": "string",
      "operationType": "string",
      "startTime": "string",
      "resourcesCount": 0,
      "successCount": 0,
      "failCount": 0,
      "resourcesStatus": [
        {
          "id": "string",
          "name": "string",
          "description": "string",
          "resourceExecutionStatus": "string",
          "status": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: wireless_settings_rf_profiles
short_description: Resource module for Wireless Settings Rf Profiles
description:
- This module represents an alias of the module wireless_settings_rf_profiles_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  defaultRfProfile:
    description: True if RF Profile is default, else False. Maximum of only 1 RF Profile
      can be marked as default at any given time.
    type: bool
  enableRadioType6GHz:
    description: True if 6 GHz radio band is enabled in the RF Profile, else False.
    type: bool
  enableRadioTypeA:
    description: True if 5 GHz radio band is enabled in the RF Profile, else False.
    type: bool
  enableRadioTypeB:
    description: True if 2.4 GHz radio band is enabled in the RF Profile, else False.
    type: bool
  id:
    description: Id path parameter. RF Profile ID.
    type: str
  radioType6GHzProperties:
    description: Wireless Settings Rf Profiles's radioType6GHzProperties.
    suboptions:
      dataRates:
        description: Data rates of 6 GHz radio band passed in comma separated format
          without any spaces. Permissible values 6, 9, 12, 18, 24, 36, 48, 54.
        type: str
      enableStandardPowerService:
        description: True if Standard Power Service is enabled, else False.
        type: bool
      mandatoryDataRates:
        description: Mandatory data rates of 6 GHz radio band passed in comma separated
          format without any spaces and must be a subset of selected dataRates with
          maximum of 2 values. Permissible values 6, 9, 12, 18, 24, 36, 48, 54.
        type: str
      maxDbsWidth:
        description: Maximum DBS Width (Permissible Values 20,40,80,160,320).
        type: int
      maxPowerLevel:
        description: Maximum power level of 6 GHz radio band.
        type: int
      minDbsWidth:
        description: Minimum DBS Width (Permissible Values 20,40,80,160,320).
        type: int
      minPowerLevel:
        description: Minimum power level of 6 GHz radio band.
        type: int
      multiBssidProperties:
        description: Wireless Settings Rf Profiles's multiBssidProperties.
        suboptions:
          dot11axParameters:
            description: Wireless Settings Rf Profiles's dot11axParameters.
            suboptions:
              muMimoDownLink:
                description: MU-MIMO Downlink.
                type: bool
              muMimoUpLink:
                description: MU-MIMO Uplink.
                type: bool
              ofdmaDownLink:
                description: OFDMA Downlink.
                type: bool
              ofdmaUpLink:
                description: OFDMA Uplink.
                type: bool
            type: dict
          dot11beParameters:
            description: Wireless Settings Rf Profiles's dot11beParameters.
            suboptions:
              muMimoDownLink:
                description: MU-MIMO Downlink.
                type: bool
              muMimoUpLink:
                description: MU-MIMO Uplink.
                type: bool
              ofdmaDownLink:
                description: OFDMA Downlink.
                type: bool
              ofdmaMultiRu:
                description: OFDMA Multi-RU.
                type: bool
              ofdmaUpLink:
                description: OFDMA Uplink.
                type: bool
            type: dict
          targetWakeTime:
            description: Target Wake Time.
            type: bool
          twtBroadcastSupport:
            description: TWT Broadcast Support.
            type: bool
        type: dict
      parentProfile:
        description: Parent profile of 6 GHz radio band.
        type: str
      powerThresholdV1:
        description: Power threshold of 6 GHz radio band.
        type: int
      preamblePuncture:
        description: Enable or Disable Preamble Puncturing. This Wifi 7 configuration
          is applicable to wireless IOS devices supporting 17.15 and higher.
        type: bool
      radioChannels:
        description: DCA channels of 6 GHz radio band passed in comma separated format
          without any spaces. Permissible values 1, 5, 9, 13, 17, 21, 25, 29, 33, 37,
          41, 45, 49, 53, 57, 61, 65, 69, 73, 77, 81, 85, 89, 93, 97, 101, 105, 109,
          113, 117, 121, 125, 129, 133, 137, 141, 145, 149, 153, 157, 161, 165, 169,
          173, 177, 181, 185, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229,
          233.
        type: str
      rxSopThreshold:
        description: RX-SOP threshold of 6 GHz radio band.
        type: str
    type: dict
  radioTypeAProperties:
    description: Wireless Settings Rf Profiles's radioTypeAProperties.
    suboptions:
      channelWidth:
        description: Channel Width.
        type: str
      dataRates:
        description: Data rates of 5 GHz radio band passed in comma separated format
          without any spaces. Permissible values 6, 9, 12, 18, 24, 36, 48, 54.
        type: str
      mandatoryDataRates:
        description: Mandatory data rates of 5 GHz radio band passed in comma separated
          format without any spaces and must be a subset of selected dataRates with
          maximum of 2 values. Permissible values 6, 9, 12, 18, 24, 36, 48, 54.
        type: str
      maxPowerLevel:
        description: Maximum power level of 5 GHz radio band.
        type: int
      minPowerLevel:
        description: Minimum power level of 5 GHz radio band.
        type: int
      parentProfile:
        description: Parent profile of 5 GHz radio band.
        type: str
      powerThresholdV1:
        description: Power threshold of 5 GHz radio band.
        type: int
      preamblePuncture:
        description: Enable or Disable Preamble Puncturing. This Wifi 7 configuration
          is applicable to wireless IOS devices supporting 17.15 and higher.
        type: bool
      radioChannels:
        description: DCA channels of 5 GHz radio band passed in comma separated format
          without any spaces. Permissible values 36, 40, 44, 48, 52, 56, 60, 64, 100,
          104, 108, 112, 116, 120, 124, 128, 132, 136, 140, 144, 149, 153, 157, 161,
          165, 169, 173.
        type: str
      rxSopThreshold:
        description: RX-SOP threshold of 5 GHz radio band.
        type: str
    type: dict
  radioTypeBProperties:
    description: Wireless Settings Rf Profiles's radioTypeBProperties.
    suboptions:
      dataRates:
        description: Data rates of 2.4 GHz radio band passed in comma separated format
          without any spaces. Permissible values 1, 2, 5.5, 6, 9, 11, 12, 18, 24, 36,
          48, 54.
        type: str
      mandatoryDataRates:
        description: Mandatory data rates of 2.4 GHz radio band passed in comma separated
          format without any spaces and must be a subset of selected dataRates with
          maximum of 2 values. Permissible values 1, 2, 5.5, 6, 9, 11, 12, 18, 24, 36,
          48, 54.
        type: str
      maxPowerLevel:
        description: Maximum power level of 2.4 GHz radio band.
        type: int
      minPowerLevel:
        description: Minimum power level of 2.4 GHz radio band.
        type: int
      parentProfile:
        description: Parent profile of 2.4 GHz radio band.
        type: str
      powerThresholdV1:
        description: Power threshold of 2.4 GHz radio band.
        type: int
      radioChannels:
        description: DCA channels of 2.4 GHz radio band passed in comma separated format
          without any spaces. Permissible values 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
          12, 13, 14.
        type: str
      rxSopThreshold:
        description: RX-SOP threshold of 2.4 GHz radio band.
        type: str
    type: dict
  rfProfileName:
    description: RF Profile Name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless CreateRFProfileV1
  description: Complete reference of the CreateRFProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-rf-profile
- name: Cisco DNA Center documentation for Wireless DeleteRFProfileV1
  description: Complete reference of the DeleteRFProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-rf-profile
- name: Cisco DNA Center documentation for Wireless UpdateRFProfileV1
  description: Complete reference of the UpdateRFProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-rf-profile
notes:
  - SDK Method used are
    wireless.Wireless.create_rf_profile_v1,
    wireless.Wireless.delete_rf_profile_v1,
    wireless.Wireless.update_rf_profile_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessSettings/rfProfiles,
    delete /dna/intent/api/v1/wirelessSettings/rfProfiles/{id},
    put /dna/intent/api/v1/wirelessSettings/rfProfiles/{id},
  - It should be noted that this module is an alias of wireless_settings_rf_profiles_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_settings_rf_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    defaultRfProfile: true
    enableRadioType6GHz: true
    enableRadioTypeA: true
    enableRadioTypeB: true
    radioType6GHzProperties:
      dataRates: string
      enableStandardPowerService: true
      mandatoryDataRates: string
      maxDbsWidth: 0
      maxPowerLevel: 0
      minDbsWidth: 0
      minPowerLevel: 0
      multiBssidProperties:
        dot11axParameters:
          muMimoDownLink: true
          muMimoUpLink: true
          ofdmaDownLink: true
          ofdmaUpLink: true
        dot11beParameters:
          muMimoDownLink: true
          muMimoUpLink: true
          ofdmaDownLink: true
          ofdmaMultiRu: true
          ofdmaUpLink: true
        targetWakeTime: true
        twtBroadcastSupport: true
      parentProfile: string
      powerThresholdV1: 0
      preamblePuncture: true
      radioChannels: string
      rxSopThreshold: string
    radioTypeAProperties:
      channelWidth: string
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      preamblePuncture: true
      radioChannels: string
      rxSopThreshold: string
    radioTypeBProperties:
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      radioChannels: string
      rxSopThreshold: string
    rfProfileName: string

- name: Delete by id
  cisco.dnac.wireless_settings_rf_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

- name: Update by id
  cisco.dnac.wireless_settings_rf_profiles:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    defaultRfProfile: true
    enableRadioType6GHz: true
    enableRadioTypeA: true
    enableRadioTypeB: true
    id: string
    radioType6GHzProperties:
      dataRates: string
      enableStandardPowerService: true
      mandatoryDataRates: string
      maxDbsWidth: 0
      maxPowerLevel: 0
      minDbsWidth: 0
      minPowerLevel: 0
      multiBssidProperties:
        dot11axParameters:
          muMimoDownLink: true
          muMimoUpLink: true
          ofdmaDownLink: true
          ofdmaUpLink: true
        dot11beParameters:
          muMimoDownLink: true
          muMimoUpLink: true
          ofdmaDownLink: true
          ofdmaMultiRu: true
          ofdmaUpLink: true
        targetWakeTime: true
        twtBroadcastSupport: true
      parentProfile: string
      powerThresholdV1: 0
      preamblePuncture: true
      radioChannels: string
      rxSopThreshold: string
    radioTypeAProperties:
      channelWidth: string
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      preamblePuncture: true
      radioChannels: string
      rxSopThreshold: string
    radioTypeBProperties:
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      radioChannels: string
      rxSopThreshold: string
    rfProfileName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: device_interface_v1_info
short_description: Information module for Device Interface V1
description:
- Get all Device Interface V1.
- Get Device Interface V1 by id.
- Returns all available interfaces. This endpoint can return a maximum of 500 interfaces.
- Returns the interface for the given interface ID.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
    - Offset query parameter.
    type: int
  limit:
    description:
    - Limit query parameter.
    type: int
  lastInputTime:
    description:
    - LastInputTime query parameter. Last Input Time.
    type: str
  lastOutputTime:
    description:
    - LastOutputTime query parameter. Last Output Time.
    type: str
  id:
    description:
    - Id path parameter. Interface ID.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetAllInterfacesV1
  description: Complete reference of the GetAllInterfacesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-all-interfaces
- name: Cisco DNA Center documentation for Devices GetInterfaceByIdV1
  description: Complete reference of the GetInterfaceByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-interface-by-id
notes:
  - SDK Method used are
    devices.Devices.get_all_interfaces_v1,
    devices.Devices.get_interface_by_id_v1,

  - Paths used are
    get /dna/intent/api/v1/interface,
    get /dna/intent/api/v1/interface/{id},

"""

EXAMPLES = r"""
- name: Get all Device Interface V1
  cisco.dnac.device_interface_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    lastInputTime: string
    lastOutputTime: string
  register: result

- name: Get Device Interface V1 by id
  cisco.dnac.device_interface_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "addresses": [
          {
            "address": {
              "ipAddress": {
                "address": "string"
              },
              "ipMask": {
                "address": "string"
              },
              "isInverseMask": true
            },
            "type": "string"
          }
        ],
        "adminStatus": "string",
        "className": "string",
        "description": "string",
        "name": "string",
        "deviceId": "string",
        "duplex": "string",
        "id": "string",
        "ifIndex": "string",
        "instanceTenantId": "string",
        "instanceUuid": "string",
        "interfaceType": "string",
        "ipv4Address": "string",
        "ipv4Mask": "string",
        "isisSupport": "string",
        "lastOutgoingPacketTime": 0,
        "lastIncomingPacketTime": 0,
        "lastUpdated": "string",
        "macAddress": "string",
        "mappedPhysicalInterfaceId": "string",
        "mappedPhysicalInterfaceName": "string",
        "mediaType": "string",
        "mtu": "string",
        "nativeVlanId": "string",
        "ospfSupport": "string",
        "pid": "string",
        "portMode": "string",
        "portName": "string",
        "portType": "string",
        "serialNo": "string",
        "series": "string",
        "speed": "string",
        "status": "string",
        "vlanId": "string",
        "voiceVlan": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2016, Pierre Jodouin <pjodouin@virtualcomputing.solutions>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lambda_event
version_added: 5.0.0
short_description: Creates, updates or deletes AWS Lambda function event mappings
description:
  - This module allows the management of AWS Lambda function event source mappings such as DynamoDB and Kinesis stream
    events via the Ansible framework. These event source mappings are relevant only in the AWS Lambda pull model, where
    AWS Lambda invokes the function.
    It is idempotent and supports "Check" mode. Use module M(amazon.aws.lambda) to manage the lambda
    function itself and M(amazon.aws.lambda_alias) to manage function aliases.
  - This module was originally added to C(community.aws) in release 1.0.0.

author:
  - Pierre Jodouin (@pjodouin)
  - Ryan Brown (@ryansb)
options:
  lambda_function_arn:
    description:
      - The name or ARN of the lambda function.
    required: true
    aliases: ['function_name', 'function_arn']
    type: str
  state:
    description:
      - Describes the desired state.
    default: "present"
    choices: ["present", "absent"]
    type: str
  alias:
    description:
      - Name of the function alias.
      - Mutually exclusive with O(version).
    type: str
  version:
    description:
      - Version of the Lambda function.
      - Mutually exclusive with O(alias).
    type: int
    default: 0
  event_source:
    description:
      - Source of the event that triggers the lambda function.
      - For DynamoDB and Kinesis events, select V(stream).
      - For SQS queues, select V(sqs).
    default: stream
    choices: ['stream', 'sqs']
    type: str
  source_params:
    description:
      - Sub-parameters required for event source.
    suboptions:
      source_arn:
        description:
          - The Amazon Resource Name (ARN) of the SQS queue, Kinesis stream or DynamoDB stream that is the event source.
        type: str
        required: true
      enabled:
        description:
          - Indicates whether AWS Lambda should begin polling or readin from the event source.
        default: true
        type: bool
      batch_size:
        description:
          - The largest number of records that AWS Lambda will retrieve from your event source at the time of invoking your function.
          - Amazon Kinesis - Default V(100). Max V(10000).
          - Amazon DynamoDB Streams - Default V(100). Max V(10000).
          - Amazon Simple Queue Service - Default V(10). For standard queues the max is V(10000). For FIFO queues the max is V(10).
          - Amazon Managed Streaming for Apache Kafka - Default V(100). Max V(10000).
          - Self-managed Apache Kafka - Default C(100). Max V(10000).
          - Amazon MQ (ActiveMQ and RabbitMQ) - Default V(100). Max V(10000).
          - DocumentDB - Default V(100). Max V(10000).
        type: int
      starting_position:
        description:
          - The position in the stream where AWS Lambda should start reading.
          - Required when O(event_source=stream).
        choices: [TRIM_HORIZON,LATEST]
        type: str
      function_response_types:
        description:
          - (Streams and Amazon SQS) A list of current response type enums applied to the event source mapping.
        type: list
        elements: str
        choices: [ReportBatchItemFailures]
        version_added: 5.5.0
      maximum_batching_window_in_seconds:
        description:
          - The maximum amount of time, in seconds, that Lambda spends gathering records before invoking the function.
          - You can configure O(source_params.maximum_batching_window_in_seconds) to any value from V(0) seconds to V(300) seconds in increments of seconds.
          - For streams and Amazon SQS event sources, when O(source_params.batch_size) is set to a value greater than V(10),
            O(source_params.maximum_batching_window_in_seconds) defaults to V(1).
          - O(source_params.maximum_batching_window_in_seconds) is not supported by FIFO queues.
        type: int
        version_added: 8.0.0
    required: true
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Example that creates a lambda event notification for a DynamoDB stream
- name: DynamoDB stream event mapping
  amazon.aws.lambda_event:
    state: present
    event_source: stream
    function_name: "{{ function_name }}"
    alias: Dev
    source_params:
      source_arn: arn:aws:dynamodb:us-east-1:123456789012:table/tableName/stream/2016-03-19T19:51:37.457
      enabled: true
      batch_size: 100
      starting_position: TRIM_HORIZON
  register: event

# Example that creates a lambda event notification for a DynamoDB stream
- name: DynamoDB stream event mapping
  amazon.aws.lambda_event:
    state: present
    event_source: stream
    function_name: "{{ function_name }}"
    source_params:
      source_arn: arn:aws:dynamodb:us-east-1:123456789012:table/tableName/stream/2016-03-19T19:51:37.457
      enabled: true
      batch_size: 100
      starting_position: LATEST
      function_response_types:
        - ReportBatchItemFailures
  register: event

- name: Show source event
  ansible.builtin.debug:
    var: event.lambda_stream_events
"""

RETURN = r"""
---
events:
    description: Dictionary returned by the API describing stream event mappings
    returned: success
    type: dict
    contains:
      batch_size:
        description: The maximum number of records in each batch that Lambda pulls.
        type: int
        returned: always
      bisect_batch_on_function_error:
        description: If the function returns an error, split the batch in two and retry.
        type: bool
        returned: for Kinesis and DynamoDB Streams only
        sample: false
      destination_config:
        description: Configuration object that specifies the destination of an event after Lambda processes it.
        type: dict
        returned: For Kinesis, DynamoDB Streams, Amazon MSK, and self-managed Kafka only
        contains:
          on_failure:
            description: The destination configuration for failed invocations.
            type: dict
            contains:
              destination:
                description: Destination resource ARN.
                type: str
          on_success:
            description: The destination configuration for failed invocations.
            type: dict
            contains:
              destination:
                description: Destination resource ARN.
                type: str
      event_source_arn:
        description: The Amazon Resource Name (ARN) of the event source.
        type: str
        returned: always
        sample: "arn:aws:dynamodb:us-west-2:123456789012:table/yyyy/stream/2024-06-06T07:03:58.956"
      function_arn:
        description: The ARN of the Lambda function.
        type: str
        returned: always
        sample: "arn:aws:lambda:us-west-2:123456789012:function:test-lambda-3a0a29d0dbb5:1"
      function_response_types:
        description: A list of current response type enums applied to the event source mapping.
        type: list
        returned: For Kinesis, DynamoDB Streams, and Amazon SQS
        sample: ["ReportBatchItemFailures"]
      last_modified:
        description: The date that the event source mapping was last updated or that its state changed.
        type: str
        returned: always
        sample: "2024-06-06T12:34:21.990000+05:30"
      last_processing_result:
        description: The result of the last Lambda invocation of your function.
        type: str
        returned: always
        sample: "No records processed"
      maximum_batching_window_in_seconds:
        description: The maximum amount of time, in seconds, that Lambda spends gathering records before invoking the function.
        type: int
        returned: always
        sample: 1
      maximum_record_age_in_seconds:
        description: Discard records older than the specified age. The default value is infinite (-1).
        type: int
        returned: For Kinesis and DynamoDB Streams only
        sample: -1
      maximum_retry_attempts:
        description: Discard records after the specified number of retries.
        type: int
        returned: For Kinesis and DynamoDB Streams only
        sample: -1
      parallelization_factor:
        description: The number of batches to process concurrently from each shard.
        type: int
        returned: For Kinesis and DynamoDB Streams only
        sample: 1
      starting_position:
        description: The position in a stream from which to start reading.
        type: str
        returned: For Kinesis and DynamoDB Streams only
        sample: "LATEST"
      state:
        description: The state of the event source mapping.
        type: str
        returned: always
        sample: "deleting"
      state_transition_reason:
        description: Indicates whether a user or Lambda made the last change to the event source mapping.
        type: str
        returned: always
        sample: "USER_INITIATED"
      tumbling_window_in_seconds:
        description: The duration in seconds of a processing window for DynamoDB and Kinesis Streams event sources.
        type: int
        returned: For Kinesis and DynamoDB Streams only
        sample: 1
      uuid:
        description: The identifier of the event source mapping.
        type: str
        returned: always
        sample: "a1b2c3d4-5678-90ab-cdef-11111EXAMPLE"
"""

import copy
import re

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
    from botocore.exceptions import MissingParametersError
    from botocore.exceptions import ParamValidationError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

# ---------------------------------------------------------------------------------------------------
#
#   Helper Functions & classes
#
# ---------------------------------------------------------------------------------------------------


def validate_params(module, client):
    """
    Performs basic parameter validation.

    :param module: The AnsibleAWSModule object
    :param client: The client used to perform requests to AWS
    :return:
    """

    function_name = module.params["lambda_function_arn"]
    qualifier = get_qualifier(module)

    # validate function name
    if not re.search(r"^[\w\-:]+$", function_name):
        module.fail_json(
            msg=f"Function name {function_name} is invalid. Names must contain only alphanumeric characters and hyphens.",
        )

    # lamba_fuction_arn contains only the function name (not the arn)
    if not function_name.startswith("arn:aws:lambda:"):
        if len(function_name) > 64:
            module.fail_json(msg=f'Function name "{function_name}" exceeds 64 character limit')
        try:
            params = {"FunctionName": function_name}
            if qualifier:
                params["Qualifier"] = qualifier
            response = client.get_function(**params)
            module.params["lambda_function_arn"] = response["Configuration"]["FunctionArn"]
        except is_boto3_error_code("ResourceNotFoundException"):
            msg = f"An error occurred: The function '{function_name}' does not exist."
            if qualifier:
                msg = f"An error occurred: The function '{function_name}' (qualifier={qualifier}) does not exist."
            module.fail_json(msg=msg)
        except ClientError as e:  # pylint: disable=duplicate-except
            module.fail_json(msg=f"An error occurred while trying to describe function '{function_name}': {e}")
    else:
        if len(function_name) > 140:
            module.fail_json(msg=f'ARN "{function_name}" exceeds 140 character limit')

        if qualifier:
            module.params["lambda_function_arn"] = f"{function_name}:{qualifier}"


def get_qualifier(module):
    """
    Returns the function qualifier as a version or alias or None.

    :param module:
    :return:
    """

    qualifier = None
    if module.params["version"] > 0:
        qualifier = str(module.params["version"])
    elif module.params["alias"]:
        qualifier = str(module.params["alias"])

    return qualifier


# ---------------------------------------------------------------------------------------------------
#
#   Lambda Event Handlers
#
#   This section defines a lambda_event_X function where X is an AWS service capable of initiating
#   the execution of a Lambda function (pull only).
#
# ---------------------------------------------------------------------------------------------------


def set_default_values(module, source_params):
    _source_params_cpy = copy.deepcopy(source_params)

    if module.params["event_source"].lower() == "sqs":
        # Default 10. For standard queues the max is 10,000. For FIFO queues the max is 10.
        _source_params_cpy.setdefault("batch_size", 10)

        if source_params["source_arn"].endswith(".fifo"):
            if _source_params_cpy["batch_size"] > 10:
                module.fail_json(msg="For FIFO queues the maximum batch_size is 10.")
            if _source_params_cpy.get("maximum_batching_window_in_seconds"):
                module.fail_json(
                    msg="maximum_batching_window_in_seconds is not supported by Amazon SQS FIFO event sources."
                )
        else:
            if _source_params_cpy["batch_size"] >= 10000:
                module.fail_json(msg="For standard queue batch_size must be between lower than 10000.")

    elif module.params["event_source"].lower() == "stream":
        # Default 100.
        _source_params_cpy.setdefault("batch_size", 100)

        if not (100 <= _source_params_cpy["batch_size"] <= 10000):
            module.fail_json(msg="batch_size for streams must be between 100 and 10000")

    if _source_params_cpy["batch_size"] > 10 and not _source_params_cpy.get("maximum_batching_window_in_seconds"):
        _source_params_cpy["maximum_batching_window_in_seconds"] = 1

    return _source_params_cpy


def lambda_event_stream(module, client):
    """
    Adds, updates or deletes lambda stream (DynamoDb, Kinesis) event notifications.
    :param module:
    :param aws:
    :return:
    """

    facts = dict()
    changed = False
    current_state = "absent"
    state = module.params["state"]

    api_params = dict(FunctionName=module.params["lambda_function_arn"])

    # check if required sub-parameters are present and valid
    source_params = module.params["source_params"]

    source_arn = source_params.get("source_arn")
    if source_arn:
        api_params.update(EventSourceArn=source_arn)
    else:
        module.fail_json(msg="Source parameter 'source_arn' is required for stream event notification.")

    if state == "present":
        source_params = set_default_values(module, source_params)

    # optional boolean value needs special treatment as not present does not imply False
    source_param_enabled = module.boolean(source_params.get("enabled", "True"))

    # check if event mapping exist
    try:
        facts = client.list_event_source_mappings(**api_params)["EventSourceMappings"]
        if facts:
            current_state = "present"
    except ClientError as e:
        module.fail_json(msg=f"Error retrieving stream event notification configuration: {e}")

    if state == "present":
        if current_state == "absent":
            starting_position = source_params.get("starting_position")
            event_source = module.params.get("event_source")
            if event_source == "stream":
                if not starting_position:
                    module.fail_json(
                        msg="Source parameter 'starting_position' is required for stream event notification."
                    )
                api_params.update(StartingPosition=starting_position)

            api_params.update(Enabled=source_param_enabled)
            if source_params.get("batch_size"):
                api_params.update(BatchSize=source_params.get("batch_size"))
            if source_params.get("maximum_batching_window_in_seconds"):
                api_params.update(
                    MaximumBatchingWindowInSeconds=source_params.get("maximum_batching_window_in_seconds")
                )
            if source_params.get("function_response_types"):
                api_params.update(FunctionResponseTypes=source_params.get("function_response_types"))

            try:
                if not module.check_mode:
                    facts = client.create_event_source_mapping(**api_params)
                changed = True
            except (ClientError, ParamValidationError, MissingParametersError) as e:
                module.fail_json(msg=f"Error creating stream source event mapping: {e}")

        else:
            # current_state is 'present'
            current_mapping = facts[0]
            api_params = dict(FunctionName=module.params["lambda_function_arn"], UUID=current_mapping["UUID"])
            mapping_changed = False

            # check if anything changed
            if source_params.get("batch_size") and source_params["batch_size"] != current_mapping["BatchSize"]:
                api_params.update(BatchSize=source_params["batch_size"])
                mapping_changed = True

            if source_param_enabled is not None:
                if source_param_enabled:
                    if current_mapping["State"] not in ("Enabled", "Enabling"):
                        api_params.update(Enabled=True)
                        mapping_changed = True
                else:
                    if current_mapping["State"] not in ("Disabled", "Disabling"):
                        api_params.update(Enabled=False)
                        mapping_changed = True

            if mapping_changed:
                try:
                    if not module.check_mode:
                        facts = client.update_event_source_mapping(**api_params)
                    changed = True
                except (ClientError, ParamValidationError, MissingParametersError) as e:
                    module.fail_json(msg=f"Error updating stream source event mapping: {e}")

    else:
        if current_state == "present":
            # remove the stream event mapping
            api_params = dict(UUID=facts[0]["UUID"])

            try:
                if not module.check_mode:
                    facts = client.delete_event_source_mapping(**api_params)
                changed = True
            except (ClientError, ParamValidationError, MissingParametersError) as e:
                module.fail_json(msg=f"Error removing stream source event mapping: {e}")

    return camel_dict_to_snake_dict(dict(changed=changed, events=facts))


def main():
    """Produce a list of function suffixes which handle lambda events."""
    source_choices = ["stream", "sqs"]

    argument_spec = dict(
        state=dict(required=False, default="present", choices=["present", "absent"]),
        lambda_function_arn=dict(required=True, aliases=["function_name", "function_arn"]),
        event_source=dict(required=False, default="stream", choices=source_choices),
        source_params=dict(
            type="dict",
            required=True,
            options=dict(
                source_arn=dict(type="str", required=True),
                enabled=dict(type="bool", default=True),
                batch_size=dict(type="int"),
                starting_position=dict(type="str", choices=["TRIM_HORIZON", "LATEST"]),
                function_response_types=dict(type="list", elements="str", choices=["ReportBatchItemFailures"]),
                maximum_batching_window_in_seconds=dict(type="int"),
            ),
        ),
        alias=dict(required=False, default=None),
        version=dict(type="int", required=False, default=0),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[["alias", "version"]],
        required_together=[],
    )

    try:
        client = module.client("lambda", retry_decorator=AWSRetry.jittered_backoff())
    except (ClientError, BotoCoreError) as e:
        module.fail_json_aws(e, msg="Trying to connect to AWS")

    validate_params(module, client)

    if module.params["event_source"].lower() in ("stream", "sqs"):
        results = lambda_event_stream(module, client)
    else:
        module.fail_json(msg="Please select `stream` or `sqs` as the event type")

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_resource_ftp
short_description: Manages resource-ftp objects on Checkpoint over Web Services API
description:
  - Manages resource-ftp objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  resource_matching_method:
    description:
      - GET allows Downloads from the server to the client. PUT allows Uploads from the client to the server.
    type: str
    choices: ['get', 'put', 'get_and_put']
  exception_track:
    description:
      - The UID or Name of the exception track to be used to log actions taken as a result of a match on the resource.
    type: str
    choices: ['none', 'exception log', 'exception alert']
  resources_path:
    description:
      - Refers to a location on the FTP server.
    type: str
  cvp:
    description:
      - Configure CVP inspection on mail messages.
    type: dict
    suboptions:
      enable_cvp:
        description:
          - Select to enable the Content Vectoring Protocol.
        type: bool
      server:
        description:
          - The UID or Name of the CVP server, make sure the CVP server is already be defined as an OPSEC Application.
        type: str
      allowed_to_modify_content:
        description:
          - Configures the CVP server to inspect but not modify content.
        type: bool
      reply_order:
        description:
          - Designates when the CVP server returns data to the Security Gateway security server.
        type: str
        choices: ['return_data_after_content_is_approved', 'return_data_before_content_is_approved']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-resource-ftp
  cp_mgmt_resource_ftp:
    exception_track: exception log
    name: newFtpResource
    resource_matching_method: get_and_put
    resources_path: path
    state: present

- name: set-resource-ftp
  cp_mgmt_resource_ftp:
    name: newFtpResource
    resource_matching_method: put
    state: present

- name: delete-resource-ftp
  cp_mgmt_resource_ftp:
    name: newFtpResource
    state: absent
"""

RETURN = """
cp_mgmt_resource_ftp:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        resource_matching_method=dict(type='str', choices=['get', 'put', 'get_and_put']),
        exception_track=dict(type='str', choices=['none', 'exception log', 'exception alert']),
        resources_path=dict(type='str'),
        cvp=dict(type='dict', options=dict(
            enable_cvp=dict(type='bool'),
            server=dict(type='str'),
            allowed_to_modify_content=dict(type='bool'),
            reply_order=dict(type='str', choices=['return_data_after_content_is_approved', 'return_data_before_content_is_approved'])
        )),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'resource-ftp'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#
# Copyright 2021 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import copy
import glob
import os

from importlib import import_module


try:
    import yaml

    # use C version if possible for speedup
    try:
        from yaml import CSafeLoader as SafeLoader
    except ImportError:
        from yaml import SafeLoader
    HAS_YAML = True
except ImportError:
    HAS_YAML = False

from ansible.errors import AnsibleActionFail, AnsibleError
from ansible.module_utils._text import to_text
from ansible.utils.display import Display

from ansible_collections.ansible.netcommon.plugins.action.network import (
    ActionModule as ActionNetworkModule,
)


display = Display()


class RunMode:
    RM_LIST = 0  # get list of supported resource modules for given os_name
    RM_GET = 1  # get resource module facts for given host
    RM_CONFIG = 1  # push resource module configuration


class ActionModule(ActionNetworkModule):
    def run(self, task_vars=None):
        run_mode = None
        self._task_vars = task_vars

        self._rm_play_context = copy.deepcopy(self._play_context)
        self._os_name = self._task.args.get("os_name") or self._get_os_name()
        if not self._os_name:
            return {"error": "either value of 'os_name' or 'ansible_network_os' should be set"}

        if len(self._os_name.split(".")) != 3:
            msg = (
                "OS value name should a fully qualified collection name in the format"
                " <org-name>.<collection-name>.<plugin-name>"
            )
            return {"error": msg}

        self._rm_play_context.network_os = self._os_name

        self._name = self._task.args.get("name")
        self._config = self._task.args.get("config")
        self._running_config = self._task.args.get("running_config")
        self._state = self._task.args.get("state")

        run_mode = self._get_run_mode()

        result = {}
        if run_mode == RunMode.RM_LIST:
            result = self._list_resource_modules()
        elif run_mode in [RunMode.RM_GET, RunMode.RM_CONFIG]:
            try:
                result = self._run_resource_module()
            except AnsibleError as exc:
                # handle short name redirection not working for ansible-2.9
                if "was not found" in to_text(exc):
                    result = self._run_resource_module(prefix_os_name=True)
                else:
                    raise

        result.update(
            {
                "ansible_network_os": self._task_vars.get("ansible_network_os"),
                "ansible_connection": self._task_vars.get("ansible_connection"),
            }
        )
        return result

    def _run_resource_module(self, prefix_os_name=False):
        new_task = self._task.copy()

        self._module = self._get_resource_module(prefix_os_name=prefix_os_name)
        if not self._module:
            msg = "Could not find resource module '%s' for os name '%s'" % (
                self._name,
                self._os_name,
            )
            raise AnsibleActionFail(msg)

        new_task.action = self._module
        action = self._shared_loader_obj.action_loader.get(
            self._rm_play_context.network_os,
            task=new_task,
            connection=self._connection,
            play_context=self._rm_play_context,
            loader=self._loader,
            templar=self._templar,
            shared_loader_obj=self._shared_loader_obj,
        )
        display.vvvv("Running resource module %s" % self._module)
        for option in ["os_name", "name"]:
            if option in new_task.args:
                new_task.args.pop(option)

        result = action.run(task_vars=self._task_vars)
        result.update({"resource_module_name": self._module})
        return result

    def _get_resource_module(self, prefix_os_name=False):
        if "." in self._name:
            if len(self._name.split(".")) != 3:
                msg = (
                    "name should a fully qualified collection name in the format"
                    " <org-name>.<collection-name>.<resource-module-name>"
                )
                raise AnsibleActionFail(msg)
            fqcn_module_name = self._name
        else:
            if prefix_os_name:
                module_name = self._os_name.split(".")[1] + "_" + self._name
            else:
                module_name = self._name

            fqcn_module_name = ".".join(self._os_name.split(".")[:2] + [module_name])

        return fqcn_module_name

    def _get_os_name(self):
        os_name = None
        if "network_os" in self._task.args and self._task.args["network_os"]:
            display.vvvv("Getting OS name from task argument")
            os_name = self._task.args["network_os"]
        elif self._play_context.network_os:
            display.vvvv("Getting OS name from inventory")
            os_name = self._play_context.network_os
        elif (
            "network_os" in self._task_vars.get("ansible_facts", {})
            and self._task_vars["ansible_facts"]["network_os"]
        ):
            display.vvvv("Getting OS name from fact")
            os_name = self._task_vars["ansible_facts"]["network_os"]

        return os_name

    def _is_resource_module(self, docs):
        doc_obj = yaml.load(docs, SafeLoader)
        if "config" in doc_obj["options"] and "state" in doc_obj["options"]:
            return True

    def _get_run_mode(self):
        error_msg = None
        if self._config or self._running_config:
            if not self._name:
                error_msg = "'name' is required if 'config' option is set"
            if not self._state:
                error_msg = "'state' is required if 'config' option is set"
            run_mode = RunMode.RM_CONFIG
        elif self._state:
            if not self._name:
                error_msg = "'name' is required if 'state' option is set"
            run_mode = RunMode.RM_GET
        elif self._name:
            if not any([self._config, self._running_config, self._state]):
                error_msg = (
                    "If 'name' is set atleast one of 'config', "
                    "'running_config' or 'state' is required"
                )
        else:
            run_mode = RunMode.RM_LIST

        if error_msg:
            raise AnsibleActionFail(error_msg)
        return run_mode

    def _list_resource_modules(self):
        result = {}
        resource_modules = []

        self._cref = dict(zip(["corg", "cname", "plugin"], self._os_name.split(".")))

        fact_modulelib = "ansible_collections.{corg}.{cname}.plugins.module_utils.network.{plugin}.facts.facts".format(
            corg=self._cref["corg"],
            cname=self._cref["cname"],
            plugin=self._cref["plugin"],
        )

        try:
            display.vvvv("fetching facts list from path %s" % (fact_modulelib))
            facts_resource_subset = getattr(import_module(fact_modulelib), "FACT_RESOURCE_SUBSETS")
            resource_modules = sorted(facts_resource_subset.keys())
        except ModuleNotFoundError:
            display.vvvv("'%s' is not defined" % (fact_modulelib))
        except AttributeError:
            display.vvvv("'FACT_RESOURCE_SUBSETS is not defined in '%s'" % (fact_modulelib))

        # parse module docs to check for 'config' and 'state' options to identify it as resource module
        if not resource_modules:
            modulelib = "ansible_collections.{corg}.{cname}.plugins.modules".format(
                corg=self._cref["corg"], cname=self._cref["cname"]
            )

            module_dir_path = os.path.dirname(import_module(modulelib).__file__)
            module_paths = glob.glob(
                "{module_dir_path}/[!_]*.py".format(module_dir_path=module_dir_path)
            )

            for module_path in module_paths:
                module_name = os.path.basename(module_path).split(".")[0]
                docs = None
                try:
                    display.vvvv("reading 'DOCUMENTATION' from path %s" % (module_path))
                    docs = getattr(
                        import_module("%s.%s" % (modulelib, module_name)),
                        "DOCUMENTATION",
                    )
                except ModuleNotFoundError:
                    display.vvvv("'%s' is not defined" % (fact_modulelib))
                except AttributeError:
                    display.vvvv("'DOCUMENTATION is not defined in '%s'" % (fact_modulelib))

                if docs:
                    if self._is_resource_module(docs):
                        resource_modules.append(module_name.split("_", 1)[1])
                    else:
                        display.vvvvv(
                            "module in path '%s' is not a resource module" % (module_path)
                        )

        result.update({"modules": sorted(resource_modules)})
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: transit_peer_network
short_description: Resource module for Transit Peer Network
description:
- This module represents an alias of the module transit_peer_network_v1
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  ipTransitSettings:
    description: Transit Peer Network's ipTransitSettings.
    suboptions:
      autonomousSystemNumber:
        description: Autonomous System Number.
        type: str
      routingProtocolName:
        description: Routing Protocol Name.
        type: str
    type: dict
  sdaTransitSettings:
    description: Transit Peer Network's sdaTransitSettings.
    suboptions:
      transitControlPlaneSettings:
        description: Transit Peer Network's transitControlPlaneSettings.
        elements: dict
        suboptions:
          deviceManagementIpAddress:
            description: Device Management Ip Address of provisioned device.
            type: str
          siteNameHierarchy:
            description: Site Name Hierarchy where device is provisioned.
            type: str
        type: list
    type: dict
  transitPeerNetworkName:
    description: TransitPeerNetworkName query parameter. Transit Peer Network Name.
    type: str
  transitPeerNetworkType:
    description: Transit Peer Network Type.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddTransitPeerNetworkV1
  description: Complete reference of the AddTransitPeerNetworkV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-transit-peer-network
- name: Cisco DNA Center documentation for SDA DeleteTransitPeerNetworkV1
  description: Complete reference of the DeleteTransitPeerNetworkV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-transit-peer-network
notes:
  - SDK Method used are
    sda.Sda.add_transit_peer_network_v1,
    sda.Sda.delete_transit_peer_network_v1,

  - Paths used are
    post /dna/intent/api/v1/business/sda/transit-peer-network,
    delete /dna/intent/api/v1/business/sda/transit-peer-network,
  - It should be noted that this module is an alias of transit_peer_network_v1

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.transit_peer_network:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    transitPeerNetworkName: string

- name: Create
  cisco.dnac.transit_peer_network:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    ipTransitSettings:
      autonomousSystemNumber: string
      routingProtocolName: string
    sdaTransitSettings:
      transitControlPlaneSettings:
      - deviceManagementIpAddress: string
        siteNameHierarchy: string
    transitPeerNetworkName: string
    transitPeerNetworkType: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: inventory
author: "Wayne Witzel III (@wwitzel3)"
short_description: create, update, or destroy Automation Platform Controller inventory.
description:
    - Create, update, or destroy Automation Platform Controller inventories. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - The name to use for the inventory.
      required: True
      type: str
    new_name:
      description:
        - Setting this option will change the existing name (looked up via the name field.
      type: str
    copy_from:
      description:
        - Name or id to copy the inventory from.
        - This will copy an existing inventory and change any parameters supplied.
        - The new inventory name will be the one provided in the name parameter.
        - The organization parameter is not used in this, to facilitate copy from one organization to another.
        - Provide the id or use the lookup plugin to provide the id if multiple inventories share the same name.
      type: str
    description:
      description:
        - The description to use for the inventory.
      type: str
    organization:
      description:
        - Organization name, ID, or named URL the inventory belongs to.
      required: True
      type: str
    variables:
      description:
        - Inventory variables.
      type: dict
    kind:
      description:
        - The kind field. Cannot be modified after created.
      choices: ["", "smart", "constructed"]
      type: str
    host_filter:
      description:
        - The host_filter field. Only useful when C(kind=smart).
      type: str
    instance_groups:
      description:
        - list of Instance Group names, IDs, or named URLs for this Organization to run on.
      type: list
      elements: str
    input_inventories:
      description:
        - List of Inventory names, IDs, or named URLs to use as input for Constructed Inventory.
      type: list
      elements: str
    prevent_instance_group_fallback:
      description:
        - Prevent falling back to instance groups set on the organization
      type: bool
    state:
      description:
        - Desired state of the resource.
      default: "present"
      choices: ["present", "absent", "exists"]
      type: str
extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: Add inventory
  inventory:
    name: "Foo Inventory"
    description: "Our Foo Cloud Servers"
    organization: "Bar Org"
    state: present
    controller_config_file: "~/tower_cli.cfg"

- name: Copy inventory
  inventory:
    name: Copy Foo Inventory
    copy_from: Default Inventory
    description: "Our Foo Cloud Servers"
    organization: Foo
    state: present

# You can create and modify constructed inventories by creating an inventory
# of kind "constructed" and then editing the automatically generated inventory
# source for that inventory.
- name: Add constructed inventory with two existing input inventories
  inventory:
    name: My Constructed Inventory
    organization: Default
    kind: constructed
    input_inventories:
      - "West Datacenter"
      - "East Datacenter"

- name: Edit the constructed inventory source
  inventory_source:
    # The constructed inventory source will always be in the format:
    # "Auto-created source for: <constructed inventory name>"
    name: "Auto-created source for: My Constructed Inventory"
    inventory: My Constructed Inventory
    limit: host3,host4,host6
    source_vars:
      plugin: constructed
      strict: true
      use_vars_plugins: true
      groups:
        shutdown: resolved_state == "shutdown"
        shutdown_in_product_dev: resolved_state == "shutdown" and account_alias == "product_dev"
      compose:
        resolved_state: state | default("running")
'''


from ..module_utils.controller_api import ControllerAPIModule
import json


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True),
        new_name=dict(),
        copy_from=dict(),
        description=dict(),
        organization=dict(required=True),
        variables=dict(type='dict'),
        kind=dict(choices=['', 'smart', 'constructed']),
        host_filter=dict(),
        instance_groups=dict(type="list", elements='str'),
        prevent_instance_group_fallback=dict(type='bool'),
        state=dict(choices=['present', 'absent', 'exists'], default='present'),
        input_inventories=dict(type='list', elements='str'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    new_name = module.params.get("new_name")
    copy_from = module.params.get('copy_from')
    description = module.params.get('description')
    organization = module.params.get('organization')
    variables = module.params.get('variables')
    state = module.params.get('state')
    kind = module.params.get('kind')
    host_filter = module.params.get('host_filter')
    prevent_instance_group_fallback = module.params.get('prevent_instance_group_fallback')

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    org_id = module.resolve_name_to_id('organizations', organization)

    # Attempt to look up inventory based on the provided name and org ID
    inventory = module.get_one('inventories', name_or_id=name, check_exists=(state == 'exists'), **{'data': {'organization': org_id}})

    # Attempt to look up credential to copy based on the provided name
    if copy_from:
        # a new existing item is formed when copying and is returned.
        inventory = module.copy_item(
            inventory,
            copy_from,
            name,
            endpoint='inventories',
            item_type='inventory',
            copy_lookup_data={},
        )

    if state == 'absent':
        # If the state was absent we can let the module delete it if needed, the module will handle exiting from this
        module.delete_if_needed(inventory)

    # Create the data that gets sent for create and update
    inventory_fields = {
        'name': new_name if new_name else (module.get_item_name(inventory) if inventory else name),
        'organization': org_id,
        'kind': kind,
        'host_filter': host_filter,
    }
    if prevent_instance_group_fallback is not None:
        inventory_fields['prevent_instance_group_fallback'] = prevent_instance_group_fallback
    if description is not None:
        inventory_fields['description'] = description
    if variables is not None:
        inventory_fields['variables'] = json.dumps(variables)

    association_fields = {}

    instance_group_names = module.params.get('instance_groups')
    if instance_group_names is not None:
        association_fields['instance_groups'] = []
        for item in instance_group_names:
            association_fields['instance_groups'].append(module.resolve_name_to_id('instance_groups', item))

    # We need to perform a check to make sure you are not trying to convert a regular inventory into a smart one.
    if inventory and inventory['kind'] == '' and inventory_fields['kind'] == 'smart':
        module.fail_json(msg='You cannot turn a regular inventory into a "smart" inventory.')

    if kind == 'constructed':
        input_inventory_names = module.params.get('input_inventories')
        if input_inventory_names is not None:
            association_fields['input_inventories'] = []
            for item in input_inventory_names:
                association_fields['input_inventories'].append(module.resolve_name_to_id('inventories', item))

    # If the state was present and we can let the module build or update the existing inventory, this will return on its own
    module.create_or_update_if_needed(
        inventory,
        inventory_fields,
        endpoint='inventories',
        item_type='inventory',
        associations=association_fields,
    )


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_virtual_account_deregister
short_description: Resource module for Pnp Virtual Account Deregister
description:
- This module represents an alias of the module pnp_virtual_account_deregister_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  domain:
    description: Domain query parameter. Smart Account Domain.
    type: str
  name:
    description: Name query parameter. Virtual Account Name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) DeregisterVirtualAccountV1
  description: Complete reference of the DeregisterVirtualAccountV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!deregister-virtual-account
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.deregister_virtual_account_v1,

  - Paths used are
    delete /dna/intent/api/v1/onboarding/pnp-settings/vacct,
  - It should be noted that this module is an alias of pnp_virtual_account_deregister_v1

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.pnp_virtual_account_deregister:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    domain: string
    name: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "smartAccountId": "string",
      "virtualAccountId": "string",
      "lastSync": 0,
      "ccoUser": "string",
      "expiry": 0,
      "autoSyncPeriod": 0,
      "profile": {
        "name": "string",
        "profileId": "string",
        "makeDefault": true,
        "addressIpV4": "string",
        "addressIpV6": "string",
        "addressFqdn": "string",
        "port": 0,
        "cert": "string",
        "proxy": true
      },
      "syncStatus": "string",
      "syncStartTime": 0,
      "tenantId": "string"
    }
"""
